"""
Test for nested class Parent

This file contains a discussion, examples, and tests about nested
classes and parents. It is kept in a separate file to avoid import
loops.

EXAMPLES:

Currently pickling fails for parents using nested classes (typically
for categories), but deriving only from Parent::

    sage: from sage.misc.nested_class_test import TestParent1, TestParent2, TestParent3, TestParent4
    sage: P = TestParent1()
    sage: TestSuite(P).run()
    Failure ...
    The following tests failed: _test_elements, _test_pickling

They actually need to be in the NestedClassMetaclass. However, due to
a technical detail, this is currently not directly supported::

    sage: P = TestParent2()
    Traceback (most recent call last):
    ...
    TypeError: metaclass conflict: the metaclass of a derived class must be a (non-strict) subclass of the metaclasses of all its bases
    sage: TestSuite(P).run()  # not tested

Instead, the easiest is to inherit from UniqueRepresentation, which is
what you want to do anyway most of the time::

    sage: P = TestParent3()
    sage: TestSuite(P).run()

This is what all Sage's parents using categories currently do. An
alternative is to use ClasscallMetaclass as metaclass::

    sage: P = TestParent4()
    sage: TestSuite(P).run()

"""
#*****************************************************************************
#  Copyright (C) 2009 Nicolas M. Thiery <nthiery at users.sf.net>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  http://www.gnu.org/licenses/
#******************************************************************************

__all__ = []  # Don't document any parents

from sage.structure.parent import Parent
from sage.structure.element_wrapper import ElementWrapper
from sage.structure.unique_representation import UniqueRepresentation
from sage.misc.classcall_metaclass import ClasscallMetaclass
from sage.misc.nested_class import NestedClassMetaclass


class TestParent1(Parent):
    def __init__(self):
        """
        EXAMPLES::

            sage: sage.misc.nested_class_test.TestParent1()
            <sage.misc.nested_class_test.TestParent1_with_category object at ...>
        """
        from sage.categories.sets_cat import Sets
        Parent.__init__(self, category = Sets())

    class Element(ElementWrapper):
        pass


class TestParent2(Parent, metaclass=NestedClassMetaclass):
    def __init__(self):
        """
        EXAMPLES::

            sage: sage.misc.nested_class_test.TestParent2()
            Traceback (most recent call last):
            ...
            TypeError: metaclass conflict: the metaclass of a derived class must be a (non-strict) subclass of the metaclasses of all its bases
        """
        from sage.categories.sets_cat import Sets
        Parent.__init__(self, category = Sets())

    class Element(ElementWrapper):
        pass


class TestParent3(UniqueRepresentation, Parent):

    def __init__(self):
        """
        EXAMPLES::

            sage: sage.misc.nested_class_test.TestParent3()
            <sage.misc.nested_class_test.TestParent3_with_category object at ...>
        """
        from sage.categories.sets_cat import Sets
        Parent.__init__(self, category = Sets())

    class Element(ElementWrapper):
        pass


class TestParent4(Parent, metaclass=ClasscallMetaclass):
    def __init__(self):
        """
        EXAMPLES::

            sage: sage.misc.nested_class_test.TestParent4()
            <sage.misc.nested_class_test.TestParent4_with_category object at ...>
        """
        from sage.categories.sets_cat import Sets
        Parent.__init__(self, category=Sets())

    def __eq__(self, other):
        """
        EXAMPLES::

            sage: from sage.misc.nested_class_test import TestParent4
            sage: TestParent4() == TestParent4()
            True
        """
        return self.__class__ == other.__class__

    def __ne__(self, other):
        """
        EXAMPLES::

            sage: from sage.misc.nested_class_test import TestParent4
            sage: TestParent4() != TestParent4()
            False
        """
        return self.__class__ != other.__class__

    def __hash__(self):
        """
        Return the hash of ``self``.

        EXAMPLES::

            sage: from sage.misc.nested_class_test import TestParent4
            sage: hash(TestParent4()) == hash(TestParent4())
            True
        """
        return hash(8960522744683456048)

    class Element(ElementWrapper):
        pass


# Class for tests:
class B(object):
    """
    A normal external class.
    """
    pass


class ABB(object):
    class B(object):
        """
        This class is broken and can't be pickled.
        A warning is emmited during compilation.
        """
        pass


class ABL(object):
    """
    There is no problem here.
    """
    B = B


class ALB(object):
    """
    There is a nested class just below. Which can't be properly sphinxed.
    """
    class C(object):
        """
        Internal C class.

        Thanks to the links below this class is pickled ok.
        But it is sphinxed wrong: It is typeset as a link to an outer class.
        """
        pass

C = ALB.C


class ABBMeta(metaclass=NestedClassMetaclass):
    class B(object):
        """
        B interne
        """
        pass


class ABLMeta(metaclass=NestedClassMetaclass):
    B = B


class ALBMeta(metaclass=NestedClassMetaclass):
    """
    There is a nested class just below which is properly sphinxed.
    """
    class CMeta(object):
        """
        B interne
        """
        pass


CMeta = ALBMeta.CMeta


class TestNestedParent(UniqueRepresentation, Parent):
    """
    This is a dummy for testing source inspection of nested classes.

    See the test in ``sage.misc.sageinspect.sage_getsourcelines``.
    """

    class Element(object):
        "This is a dummy element class"
        pass



"""
Mutability -- Pyrex Implementation
"""

##########################################################################
#
#   Sage: Open Source Mathematical Software
#
#       Copyright (C) 2006 William Stein <wstein@ucsd.edu>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  https://www.gnu.org/licenses/
##########################################################################

cdef class Mutability:
    cdef public bint _is_immutable
    cpdef _require_mutable(self)
    cpdef _require_immutable(self)
    cpdef bint is_immutable(self)
    cpdef bint is_mutable(self)


# Copyright (c) 2009, Tom Boothby <boothby@math.washington.edu>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of Sage nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY Tom Boothby ''AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Tom Boothby BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""

Function Mangling

This module provides utilities for extracting information about python
functions.

AUTHORS:

- Tom Boothby (2009): Original version in Python
- Simon King (2011): Use Cython. Speedup of ``fix_to_pos``, cleaning documentation.

"""

from sage.misc.sageinspect import sage_getargspec

cdef class ArgumentFixer:
    """
    This class provides functionality to normalize the arguments
    passed into a function.  While the various ways of calling a
    function are perfectly equivalent from the perspective of the
    callee, they don't always look the same for an object
    watching the caller.  For example,
    ::

        sage: def f(x = 10):
        ....:     return min(1,x)

    the following calls are equivalent,
    ::

        sage: f()
        1
        sage: f(10)
        1
        sage: f(x=10)
        1

    but from the perspective of a wrapper, they are different::

        sage: def wrap(g):
        ....:     def _g(*args,**kwargs):
        ....:         print("{} {}".format(args, kwargs))
        ....:         return g(*args, **kwargs)
        ....:     return _g
        sage: h = wrap(f)
        sage: t = h()
        () {}
        sage: t = h(10)
        (10,) {}
        sage: t = h(x=10)
        () {'x': 10}

    For the purpose of cached functions, it is important not
    to distinguish between these uses.

    INPUT:

    - f           -- a function
    - classmethod -- boolean (default False) -- True if the function
      is a classmethod and therefore the first
      argument is expected to be the class instance.
      In that case, we ignore the first argument.

    EXAMPLES::

        sage: from sage.misc.function_mangling import ArgumentFixer
        sage: def wrap2(g):
        ....:     af = ArgumentFixer(g)
        ....:     def _g(*args, **kwargs):
        ....:         print(af.fix_to_pos())
        ....:         return g(*args,**kwargs)
        ....:     return _g
        sage: h2 = wrap2(f)
        sage: t = h2()
        ((10,), ())
        sage: t = h2(10)
        ((10,), ())
        sage: t = h2(x=10)
        ((10,), ())

    ::

        sage: class one:
        ....:    def __init__(self, x = 1):
        ....:       self.x = x
        sage: af = ArgumentFixer(one.__init__, classmethod=True)
        sage: af.fix_to_pos(1,2,3,a=31,b=2,n=3)
        ((1, 2, 3), (('a', 31), ('b', 2), ('n', 3)))

    """
    def __init__(self, f, classmethod = False):
        try:
            arg_names, varargs, varkw, defaults = sage_getargspec(f)
        except AttributeError:
            # This error occurs if f is defined in a Cython file and the
            # source file has gone.
            if classmethod:
                arg_names = ['self']
                varargs = 'args'
                varkws = 'kwds'
                defaults = None
            else:
                arg_names = []
                varargs = 'args'
                varkws = 'kwds'
                defaults = None
        if defaults is None:
            self._default_tuple = defaults = ()
        else:
            self._default_tuple = tuple(defaults)

        #code = f.__code__

        self.f = f
        self._ndefault = len(defaults)
        if classmethod:
            self._nargs = len(arg_names)-1 #code.co_argcount-1
            self._arg_names = tuple(arg_names[1:]) #code.co_varnames[1:self._nargs+1]
        else:
            self._nargs = len(arg_names) #code.co_argcount
            self._arg_names = tuple(arg_names) #code.co_varnames[:self._nargs]
        self._classmethod = classmethod

        cdef dict default_map
        self._defaults = default_map = {}
        for k,v in zip(self._arg_names[-self._ndefault:], defaults):
            default_map[k] = v

    def __repr__(self):
        """
        EXAMPLES::

            sage: from sage.misc.function_mangling import ArgumentFixer
            sage: g = ArgumentFixer(number_of_partitions)
            sage: g
            Argument Fixer of <function number_of_partitions at 0x...>
        """
        return "Argument Fixer of %s"%self.f

    def fix_to_named(self, *args,**kwargs):
        """
        Normalize the arguments with a preference for named arguments.

        INPUT:

        - any positional and named arguments.

        OUTPUT:

        We return a tuple

            `(e_1, e_2, ..., e_k), ((n_1, v_1), ... , (n_m, v_m))`

        where `n_1, ... , n_m` are the names of the arguments and
        `v_1, ..., v_m` are the values passed in; and `e_1, ..., e_k` are
        the unnamed arguments.  We minimize `k`.

        The defaults are extracted from the function and filled
        into the list ``K`` of named arguments. The names `n_1, ..., n_t`
        are in order of the function definition, where `t` is the number
        of named arguments.  The remaining names, `n_{t+1}, ..., n_m` are
        given in alphabetical order.  This is useful to extract
        the names of arguments, but **does not** maintain
        equivalence of
        ::

            A,K = self.fix_to_pos(...)
            self.f(*A,**dict(K))`

        and
        ::

            self.f(...)

        in all cases.

        EXAMPLES::

            sage: from sage.misc.function_mangling import ArgumentFixer
            sage: def sum3(a,b,c=3,*args,**kwargs):
            ....:     return a+b+c
            sage: AF = ArgumentFixer(sum3)
            sage: AF.fix_to_named(1,2,3,4,5,6,f=14,e=16)
            ((4, 5, 6), (('a', 1), ('b', 2), ('c', 3), ('e', 16), ('f', 14)))
            sage: AF.fix_to_named(1,2,f=14)
            ((), (('a', 1), ('b', 2), ('c', 3), ('f', 14)))

        """
        cdef list ARGS = []
        cdef tuple arg_names = self._arg_names
        cdef int lenargs = len(args)
        cdef dict defaults = self._defaults
        cdef int i
        cdef dict kwargs_ = dict(kwargs)
        for i from 0<=i<self._nargs:
            name = arg_names[i]
            if i >= lenargs:
                if name in kwargs_:
                    val = kwargs_[name]
                    del kwargs_[name]
                else:
                    val = defaults[name]
            else:
                val = args[i]
            ARGS.append((name,val))
        extra_args = args[self._nargs:]
        for k in sorted(kwargs_.keys()):
            ARGS.append((k,kwargs_[k]))
        return tuple(extra_args), tuple(ARGS)

    def fix_to_pos(self, *args, **kwds):
        """
        Normalize the arguments with a preference for positional arguments.

        INPUT:

        Any positional or named arguments

        OUTPUT:

        We return a tuple

            `(e_1, e_2, ..., e_k), ((n_1, v_1), ... , (n_m, v_m))`

        where `n_1, ... , n_m` are the names of the arguments and
        `v_1, ..., v_m` are the values passed in; and `e_1, ..., e_k`
        are the unnamed arguments. We minimize `m`.

        The commands
        ::

            A,K = self.fix_to_pos(...)
            self.f(*A,**dict(K))

        are equivalent to
        ::

            self.f(...)

        though defaults are extracted from the function and
        appended to the tuple ``A`` of positional arguments.
        The names `n_1, ..., n_m` are given in alphabetical
        order.

        EXAMPLES::

            sage: from sage.misc.function_mangling import ArgumentFixer
            sage: def do_something(a, b, c=3, *args, **kwargs):
            ....:     print("{} {} {} {} {}".format(a, b, c, args,
            ....:                                   sorted(kwargs.items())))
            sage: AF = ArgumentFixer(do_something)
            sage: A, K = AF.fix_to_pos(1, 2, 3, 4, 5, 6, f=14, e=16)
            sage: print("{} {}".format(A, K))
            (1, 2, 3, 4, 5, 6) (('e', 16), ('f', 14))
            sage: do_something(*A, **dict(K))
            1 2 3 (4, 5, 6) [('e', 16), ('f', 14)]
            sage: do_something(1, 2, 3, 4, 5, 6, f=14, e=16)
            1 2 3 (4, 5, 6) [('e', 16), ('f', 14)]
        """
        return self.fix_to_pos_args_kwds(args, kwds)

    cdef fix_to_pos_args_kwds(self, tuple args, dict kwds):
        """
        Fast Cython implementation of :meth:`fix_to_pos`.
        """
        cdef Py_ssize_t lenargs = len(args)
        cdef Py_ssize_t nargs = self._nargs
        cdef tuple arg_names = self._arg_names
        cdef dict defaults = self._defaults
        # a shortpath for the case of no named arguments:
        if not kwds:
            if lenargs >= nargs:
                return args, ()
            # we take the given arguments, plus the default arguments
            return args + self._default_tuple[-nargs+lenargs:],()
        cdef list Largs = list(args)
        cdef dict kwargs = dict(kwds)
        cdef Py_ssize_t i
        for i in range(lenargs, nargs):
            # in addition to the positional arguments, we take the
            # ones with default values, unless they are overridded by
            # the named arguments.
            name = arg_names[i]
            if name in kwargs:
                val = kwargs[name]
                del kwargs[name]
            else:
                val = defaults[name]
            Largs.append(val)
        cdef list Items = sorted(kwargs.items())
        return tuple(Largs), tuple(Items)



"""
Realizations Covariant Functorial Construction

.. SEEALSO::

    - :func:`Sets().WithRealizations <sage.categories.with_realizations.WithRealizations>`
      for an introduction to *realizations* and *with realizations*.
    - :mod:`sage.categories.covariant_functorial_construction`
      for an introduction to covariant functorial constructions.
    - :mod:`sage.categories.examples.with_realizations` for an example.
"""
#*****************************************************************************
#  Copyright (C) 2010-2012 Nicolas M. Thiery <nthiery at users.sf.net>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  http://www.gnu.org/licenses/
#*****************************************************************************

from sage.misc.bindable_class import BindableClass
from sage.categories.category import Category
from sage.categories.category_types import Category_over_base
from sage.categories.covariant_functorial_construction import RegressiveCovariantConstructionCategory

class RealizationsCategory(RegressiveCovariantConstructionCategory):
    """
    An abstract base class for all categories of realizations category

    Relization are implemented as
    :class:`~sage.categories.covariant_functorial_construction.RegressiveCovariantConstructionCategory`.
    See there for the documentation of how the various bindings such
    as ``Sets().Realizations()`` and ``P.Realizations()``, where ``P``
    is a parent, work.

    .. SEEALSO:: :func:`Sets().WithRealizations <sage.categories.with_realizations.WithRealizations>`

    TESTS::

        sage: Sets().Realizations
        <bound method Realizations of Category of sets>
        sage: Sets().Realizations()
        Category of realizations of sets
        sage: Sets().Realizations().super_categories()
        [Category of sets]
        sage: Groups().Realizations().super_categories()
        [Category of groups, Category of realizations of unital magmas]
    """

    _functor_category = "Realizations"

def Realizations(self):
    """
    Return the category of realizations of the parent ``self`` or of objects
    of the category ``self``

    INPUT:

    - ``self`` -- a parent or a concrete category

    .. NOTE:: this *function* is actually inserted as a *method* in the class
       :class:`~sage.categories.category.Category` (see
       :meth:`~sage.categories.category.Category.Realizations`). It is defined
       here for code locality reasons.

    EXAMPLES:

    The category of realizations of some algebra::

        sage: Algebras(QQ).Realizations()
        Join of Category of algebras over Rational Field and Category of realizations of unital magmas

    The category of realizations of a given algebra::

        sage: A = Sets().WithRealizations().example(); A
        The subset algebra of {1, 2, 3} over Rational Field
        sage: A.Realizations()
        Category of realizations of The subset algebra of {1, 2, 3} over Rational Field

        sage: C = GradedHopfAlgebrasWithBasis(QQ).Realizations(); C
        Join of Category of graded hopf algebras with basis over Rational Field and Category of realizations of hopf algebras over Rational Field
        sage: C.super_categories()
        [Category of graded hopf algebras with basis over Rational Field, Category of realizations of hopf algebras over Rational Field]

        sage: TestSuite(C).run()

    .. SEEALSO::

        - :func:`Sets().WithRealizations <sage.categories.with_realizations.WithRealizations>`
        - :class:`ClasscallMetaclass`

    .. TODO::

        Add an optional argument to allow for::

            sage: Realizations(A, category = Blahs()) # todo: not implemented
    """
    if isinstance(self, Category):
        return RealizationsCategory.category_of(self)
    else:
        return getattr(self.__class__, "Realizations")(self)

Category.Realizations = Realizations

class Category_realization_of_parent(Category_over_base, BindableClass):
    """
    An abstract base class for categories of all realizations of a given parent

    INPUT:

    - ``parent_with_realization`` -- a parent

    .. SEEALSO:: :func:`Sets().WithRealizations <sage.categories.with_realizations.WithRealizations>`

    EXAMPLES::

        sage: A = Sets().WithRealizations().example(); A
        The subset algebra of {1, 2, 3} over Rational Field

    The role of this base class is to implement some technical goodies, like
    the binding ``A.Realizations()`` when a subclass ``Realizations`` is
    implemented as a nested class in ``A``
    (see the :mod:`code of the example <sage.categories.examples.with_realizations.SubsetAlgebra>`)::

        sage: C = A.Realizations(); C
        Category of realizations of The subset algebra of {1, 2, 3} over Rational Field

    as well as the name for that category.
    """
    def __init__(self, parent_with_realization):
        """
        TESTS::

            sage: from sage.categories.realizations import Category_realization_of_parent
            sage: A = Sets().WithRealizations().example(); A
            The subset algebra of {1, 2, 3} over Rational Field
            sage: C = A.Realizations(); C
            Category of realizations of The subset algebra of {1, 2, 3} over Rational Field
            sage: isinstance(C, Category_realization_of_parent)
            True
            sage: C.parent_with_realization
            The subset algebra of {1, 2, 3} over Rational Field
            sage: TestSuite(C).run(skip=["_test_category_over_bases"])

        .. TODO::

            Fix the failing test by making ``C`` a singleton
            category. This will require some fiddling with the
            assertion in :meth:`Category_singleton.__classcall__`
        """
        Category_over_base.__init__(self, parent_with_realization)
        self.parent_with_realization = parent_with_realization

    def _get_name(self):
        """
        Return a human readable string specifying which kind of bases this category is for

        It is obtained by splitting and lower casing the last part of
        the class name.

        EXAMPLES::

            sage: from sage.categories.realizations import Category_realization_of_parent
            sage: class MultiplicativeBasesOnPrimitiveElements(Category_realization_of_parent):
            ....:     def super_categories(self): return [Objects()]
            sage: Sym = SymmetricFunctions(QQ); Sym.rename("Sym")
            sage: MultiplicativeBasesOnPrimitiveElements(Sym)._get_name()
            'multiplicative bases on primitive elements'
        """
        import re
        return re.sub(".[A-Z]", lambda s: s.group()[0]+" "+s.group()[1], self.__class__.__base__.__name__.split(".")[-1]).lower()

    def _repr_object_names(self):
        """
        Return the name of the objects of this category.

        .. SEEALSO:: :meth:`Category._repr_object_names`

        EXAMPLES::

            sage: from sage.categories.realizations import Category_realization_of_parent
            sage: class MultiplicativeBasesOnPrimitiveElements(Category_realization_of_parent):
            ....:     def super_categories(self): return [Objects()]
            sage: Sym = SymmetricFunctions(QQ); Sym.rename("Sym")
            sage: C = MultiplicativeBasesOnPrimitiveElements(Sym); C
            Category of multiplicative bases on primitive elements of Sym
            sage: C._repr_object_names()
            'multiplicative bases on primitive elements of Sym'
        """
        return "{} of {}".format(self._get_name(), self.base())




r"""
Unit testing for Sage objects
"""

# ****************************************************************************
#       Copyright (C) 2009 Nicolas M. Thiery <nthiery at users.sf.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#                  https://www.gnu.org/licenses/
# ****************************************************************************

import unittest
import sys
import traceback


class TestSuite(object):
    """
    Test suites for Sage objects.

    EXAMPLES::

        sage: TestSuite(ZZ).run()

    No output means that all tests passed. Which tests?
    In practice this calls all the methods ``._test_*`` of this
    object, in alphabetic order::

        sage: TestSuite(1).run(verbose = True)
        running ._test_category() . . . pass
        running ._test_eq() . . . pass
        running ._test_new() . . . pass
        running ._test_nonzero_equal() . . . pass
        running ._test_not_implemented_methods() . . . pass
        running ._test_pickling() . . . pass

    Those methods are typically implemented by abstract
    super classes, in particular via categories, in order to
    enforce standard behavior and API, or provide mathematical
    sanity checks. For example if ``self`` is in the category of
    finite semigroups, this checks that the multiplication is
    associative (at least on some elements)::

        sage: S = FiniteSemigroups().example(alphabet = ('a', 'b'))
        sage: TestSuite(S).run(verbose = True)
        running ._test_an_element() . . . pass
        running ._test_associativity() . . . pass
        running ._test_cardinality() . . . pass
        running ._test_category() . . . pass
        running ._test_construction() . . . pass
        running ._test_elements() . . .
          Running the test suite of self.an_element()
          running ._test_category() . . . pass
          running ._test_eq() . . . pass
          running ._test_new() . . . pass
          running ._test_not_implemented_methods() . . . pass
          running ._test_pickling() . . . pass
          pass
        running ._test_elements_eq_reflexive() . . . pass
        running ._test_elements_eq_symmetric() . . . pass
        running ._test_elements_eq_transitive() . . . pass
        running ._test_elements_neq() . . . pass
        running ._test_enumerated_set_contains() . . . pass
        running ._test_enumerated_set_iter_cardinality() . . . pass
        running ._test_enumerated_set_iter_list() . . . pass
        running ._test_eq() . . . pass
        running ._test_new() . . . pass
        running ._test_not_implemented_methods() . . . pass
        running ._test_pickling() . . . pass
        running ._test_some_elements() . . . pass

    The different test methods can be called independently::

        sage: S._test_associativity()

    Debugging tip: in case of failure of some test, use ``%pdb on`` to
    turn on automatic debugging on error. Run the failing test
    independently: the debugger will stop right where the first
    assertion fails. Then, introspection can be used to analyse what
    exactly the problem is. See also the ``catch = False`` option to
    :meth:`.run`.

    When meaningful, one can further customize on which elements
    the tests are run. Here, we use it to *prove* that the
    multiplication is indeed associative, by running the test on
    all the elements::

        sage: S._test_associativity(elements = S)

    Adding a new test boils down to adding a new method in the class
    of the object or any super class (e.g. in a category). This method
    should use the utility :meth:`._tester` to handle standard options
    and report test failures. See the code of
    :meth:`._test_an_element` for an example. Note: Python's testunit
    convention is to look for methods called ``.test*``; we use instead
    ``._test_*`` so as not to pollute the object's interface.

    Eventually, every implementation of a :class:`SageObject` should
    run a :class:`TestSuite` on one of its instances in its doctest
    (replacing the current ``loads(dumps(x))`` tests).

    Finally, running ``TestSuite`` on a standard Python object does
    some basic sanity checks::

        sage: TestSuite(int(1)).run(verbose = True)
        running ._test_new() . . . pass
        running ._test_pickling() . . . pass

    TODO:

     - Allow for customized behavior in case of failing assertion
       (warning, error, statistic accounting).
       This involves reimplementing the methods fail / failIf / ...
       of unittest.TestCase in InstanceTester

     - Don't catch the exceptions if ``TestSuite(..).run()`` is called
       under the debugger, or with ``%pdb`` on (how to detect this? see
       ``get_ipython()``, ``IPython.Magic.shell.call_pdb``, ...)
       In the mean time, see the ``catch=False`` option.

     - Run the tests according to the inheritance order, from most
       generic to most specific, rather than alphabetically. Then, the
       first failure will be the most relevant, the others being
       usually consequences.

     - Improve integration with doctests (statistics on failing/passing tests)

     - Add proper support for nested testsuites.

     - Integration with unittest:
       Make TestSuite inherit from unittest.TestSuite?
       Make ``.run(...)`` accept a result object

     - Add some standard option ``proof = True``, asking for the
       test method to choose appropriately the elements so as to
       prove the desired property. The test method may assume that
       a parent implements properly all the super categories. For
       example, the ``_test_commutative`` method of the category
       ``CommutativeSemigroups()`` may just check that the
       provided generators commute, implicitly assuming that
       generators indeed generate the semigroup (as required by
       ``Semigroups()``).
    """

    def __init__(self, instance):
        """
        TESTS::

            sage: TestSuite(ZZ)
            Test suite for Integer Ring
        """
        from sage.structure.sage_object import SageObject
        if not isinstance(instance, (SageObject, PythonObjectWithTests)):
            instance = PythonObjectWithTests(instance)
        self._instance = instance

    def __repr__(self):
        """
        TESTS::

            sage: TestSuite(ZZ)
            Test suite for Integer Ring
        """
        return "Test suite for %s" % self._instance

    def run(self, category=None, skip=[], catch=True, raise_on_failure=False,
            **options):
        """
        Run all the tests from this test suite:

        INPUT:

         - ``category``         - a category; reserved for future use
         - ``skip``             - a string or list (or iterable) of strings
         - ``raise_on_failure`` - a boolean (default: False)
         - ``catch``            - a boolean (default: True)

        All other options are passed down to the individual tests.

        EXAMPLES::

            sage: TestSuite(ZZ).run()

        We now use the ``verbose`` option::

            sage: TestSuite(1).run(verbose = True)
            running ._test_category() . . . pass
            running ._test_eq() . . . pass
            running ._test_new() . . . pass
            running ._test_nonzero_equal() . . . pass
            running ._test_not_implemented_methods() . . . pass
            running ._test_pickling() . . . pass

        Some tests may be skipped using the ``skip`` option::

            sage: TestSuite(1).run(verbose = True, skip ="_test_pickling")
            running ._test_category() . . . pass
            running ._test_eq() . . . pass
            running ._test_new() . . . pass
            running ._test_nonzero_equal() . . . pass
            running ._test_not_implemented_methods() . . . pass
            sage: TestSuite(1).run(verbose = True, skip =["_test_pickling", "_test_category"])
            running ._test_eq() . . . pass
            running ._test_new() . . . pass
            running ._test_nonzero_equal() . . . pass
            running ._test_not_implemented_methods() . . . pass

        We now show (and test) some standard error reports::

            sage: class Blah(SageObject):
            ....:     def _test_a(self, tester): pass
            ....:     def _test_b(self, tester): tester.fail()
            ....:     def _test_c(self, tester): pass
            ....:     def _test_d(self, tester): tester.fail()

            sage: TestSuite(Blah()).run()
            Failure in _test_b:
            Traceback (most recent call last):
              ...
            AssertionError: None
            ------------------------------------------------------------
            Failure in _test_d:
            Traceback (most recent call last):
              ...
            AssertionError: None
            ------------------------------------------------------------
            Failure in _test_pickling:
            Traceback (most recent call last):
              ...
            ...PicklingError: Can't pickle <class '__main__.Blah'>: attribute
            lookup ...Blah... failed
            ------------------------------------------------------------
            The following tests failed: _test_b, _test_d, _test_pickling

            sage: TestSuite(Blah()).run(verbose = True)
            running ._test_a() . . . pass
            running ._test_b() . . . fail
            Traceback (most recent call last):
              ...
            AssertionError: None
            ------------------------------------------------------------
            running ._test_c() . . . pass
            running ._test_category() . . . pass
            running ._test_d() . . . fail
            Traceback (most recent call last):
              ...
            AssertionError: None
            ------------------------------------------------------------
            running ._test_new() . . . pass
            running ._test_not_implemented_methods() . . . pass
            running ._test_pickling() . . . fail
            Traceback (most recent call last):
              ...
            ...PicklingError: Can't pickle <class '__main__.Blah'>: attribute
            lookup ...Blah... failed
            ------------------------------------------------------------
            The following tests failed: _test_b, _test_d, _test_pickling

            File "/opt/sage/local/lib/python/site-packages/sage/misc/sage_unittest.py", line 183, in run
            test_method(tester = tester)

        The ``catch=False`` option prevents ``TestSuite`` from
        catching exceptions::

            sage: TestSuite(Blah()).run(catch=False)
            Traceback (most recent call last):
              ...
              File ..., in _test_b
                def _test_b(self, tester): tester.fail()
              ...
            AssertionError: None

        In conjunction with ``%pdb on``, this allows for the debugger
        to jump directly to the first failure location.
        """
        if isinstance(skip, str):
            skip = [skip]
        else:
            skip = tuple(skip)

        # The class of exceptions that will be caught and reported;
        # other exceptions will get through. () catches nothing.
        catch_exception = Exception if catch else ()

        tester = instance_tester(self._instance, **options)
        failed = []
        for method_name in dir(self._instance):
            if method_name[0:6] == "_test_" and method_name not in skip:
                # TODO: improve pretty printing
                # could use the doc string of the test method?
                tester.info(tester._prefix + "running .%s() . . ." % method_name, newline=False)
                test_method = getattr(self._instance, method_name)
                try:
                    test_method(tester=tester)
                    tester.info(" pass")
                except catch_exception as e:
                    failed.append(method_name)
                    if isinstance(e, TestSuiteFailure):
                        # The failure occurred in a nested testsuite
                        # which has already reported the details of
                        # that failure
                        if not tester._verbose:
                            print(tester._prefix + "Failure in {}".format(method_name))
                    else:
                        if tester._verbose:
                            tester.info(" fail")
                        else:
                            print(tester._prefix + "Failure in {}:".format(method_name))
                        s = traceback.format_exc()
                        print(tester._prefix + s.strip().replace("\n", "\n" + tester._prefix))
                        print(tester._prefix + "-" * 60)
        if failed:
            print(tester._prefix + "The following tests failed: {}".format(", ".join(failed)))
            if raise_on_failure:
                raise TestSuiteFailure


class TestSuiteFailure(AssertionError):
    pass


def instance_tester(instance, tester=None, **options):
    """
    Return a gadget attached to ``instance`` providing testing utilities.

    EXAMPLES::

        sage: from sage.misc.sage_unittest import instance_tester
        sage: tester = instance_tester(ZZ)

        sage: tester.assertTrue(1 == 1)
        sage: tester.assertTrue(1 == 0)
        Traceback (most recent call last):
        ...
        AssertionError: False is not true
        sage: tester.assertTrue(1 == 0, "this is expected to fail")
        Traceback (most recent call last):
        ...
        AssertionError: this is expected to fail

        sage: tester.assertEqual(1, 1)
        sage: tester.assertEqual(1, 0)
        Traceback (most recent call last):
        ...
        AssertionError: 1 != 0

    The available assertion testing facilities are the same as in
    :class:`unittest.TestCase` [UNITTEST]_, which see (actually, by a slight
    abuse, tester is currently an instance of this class).

    TESTS::

        sage: instance_tester(ZZ, tester = tester) is tester
        True
    """
    if tester is None:
        return InstanceTester(instance, **options)
    else:
        assert not options
        assert tester._instance is instance
        return tester


class InstanceTester(unittest.TestCase):
    """
    A gadget attached to an instance providing it with testing utilities.

    EXAMPLES::

        sage: from sage.misc.sage_unittest import InstanceTester
        sage: InstanceTester(instance = ZZ, verbose = True, elements = [1,2,3])
        Testing utilities for Integer Ring

    This is used by ``SageObject._tester``, which see::

        sage: QQ._tester()
        Testing utilities for Rational Field
    """

    # On Python 3 this attribute defaults to True, causing the AssertionErrors
    # output by failed test cases to produce longer error messages than the
    # default error messages on Python 2.  So for backwards compatibility of
    # existing test cases we disable these "long messages" (which don't gain us
    # all that much anyways)
    longMessage = False

    def __init__(self, instance, elements=None, verbose=False, prefix="",
                 max_runs=4096, max_samples=None, **options):
        """
        A gadget attached to an instance providing it with testing utilities.

        EXAMPLES::

            sage: from sage.misc.sage_unittest import InstanceTester
            sage: InstanceTester(instance = ZZ, verbose = True, elements = [1,2,3])
            Testing utilities for Integer Ring

        This is used by ``SageObject._tester``, for example::

            sage: QQ._tester()
            Testing utilities for Rational Field
        """
        unittest.TestCase.__init__(self)
        self._instance = instance
        self._verbose = verbose
        self._elements = elements
        self._prefix = prefix
        self._max_runs = max_runs
        self._max_samples = max_samples

    def runTest(self):
        """
        Trivial implementation of :meth:`unittest.TestCase.runTest` to
        please the super class :class:`TestCase`. That's the price to
        pay for abusively inheriting from it.

        EXAMPLES::

            sage: from sage.misc.sage_unittest import InstanceTester
            sage: tester = InstanceTester(ZZ, verbose = True)
            sage: tester.runTest()
        """
        pass

    def info(self, message, newline=True):
        """
        Display user information

        EXAMPLES::

            sage: from sage.misc.sage_unittest import InstanceTester
            sage: tester = InstanceTester(ZZ, verbose = True)

            sage: tester.info("hello"); tester.info("world")
            hello
            world

            sage: tester = InstanceTester(ZZ, verbose = False)
            sage: tester.info("hello"); tester.info("world")

            sage: tester = InstanceTester(ZZ, verbose = True)
            sage: tester.info("hello", newline = False); tester.info(" world")
            hello world
        """
        if self._verbose:
            if newline:
                sys.stdout.write(message + "\n")
            else:
                sys.stdout.write(message)
            sys.stdout.flush()

    def __repr__(self):
        """
        EXAMPLES::

            sage: from sage.misc.sage_unittest import InstanceTester
            sage: InstanceTester(ZZ, verbose = True)
            Testing utilities for Integer Ring

        """
        return "Testing utilities for %s" % self._instance

    def some_elements(self, S=None, repeat=None):
        """
        Return a list (or iterable) of elements of the instance on which
        the tests should be run.

        This is only meaningful for container objects like parents.

        INPUT:

        - ``S`` -- a set of elements to select from.  By default this
          will use the elements passed to this tester at creation
          time, or the result of :meth:`.some_elements` if no elements
          were specified.

        - ``repeat`` -- integer (default: None).  If given, instead returns
          a list of tuples of length ``repeat`` from ``S``.

        OUTPUT:

        A list of at most ``self._max_runs`` elements of ``S^r``,
        or a sample of at most ``self._max_samples`` if that is not ``None``.

        EXAMPLES:

        By default, this calls :meth:`.some_elements` on the instance::

            sage: from sage.misc.sage_unittest import InstanceTester
            sage: class MyParent(Parent):
            ....:     def some_elements(self):
            ....:         return [1,2,3,4,5]
            ...
            sage: tester = InstanceTester(MyParent())
            sage: list(tester.some_elements())
            [1, 2, 3, 4, 5]

            sage: tester = InstanceTester(MyParent(), max_runs=3)
            sage: list(tester.some_elements())
            [1, 2, 3]

            sage: tester = InstanceTester(MyParent(), max_runs=7)
            sage: list(tester.some_elements())
            [1, 2, 3, 4, 5]

            sage: tester = InstanceTester(MyParent(), elements=[1,3,5])
            sage: list(tester.some_elements())
            [1, 3, 5]

            sage: tester = InstanceTester(MyParent(), elements=[1,3,5], max_runs=2)
            sage: list(tester.some_elements())
            [1, 3]

            sage: tester = InstanceTester(FiniteEnumeratedSet(['a','b','c','d']), max_runs=3)
            sage: tester.some_elements()
            ['a', 'b', 'c']

            sage: tester = InstanceTester(FiniteEnumeratedSet([]))
            sage: list(tester.some_elements())
            []

            sage: tester = InstanceTester(ZZ)
            sage: ZZ.some_elements()             # yikes, shamelessly trivial ...
            <generator object ..._some_elements_from_iterator at 0x...>
            sage: list(tester.some_elements())
            [0, 1, -1, 2, -2, ..., 49, -49, 50]

            sage: tester = InstanceTester(ZZ, elements = ZZ, max_runs=5)
            sage: list(tester.some_elements())
            [0, 1, -1, 2, -2]

            sage: tester = InstanceTester(ZZ, elements = srange(100), max_runs=5)
            sage: list(tester.some_elements())
            [0, 1, 2, 3, 4]

            sage: tester = InstanceTester(ZZ, elements = srange(3), max_runs=5)
            sage: list(tester.some_elements())
            [0, 1, 2]

        The ``repeat`` keyword can give pairs or triples from ``S``::

            sage: list(tester.some_elements(repeat=2))
            [(0, 0), (0, 1), (0, 2), (1, 0), (1, 1)]

        You can use ``max_samples`` to sample at random, instead of in order::

            sage: tester = InstanceTester(ZZ, elements = srange(8), max_samples = 4)
            sage: all(t in srange(8) for t in tester.some_elements())
            True
            sage: all(s in srange(8) and t in srange(8) for s,t in tester.some_elements(repeat=2))
            True

        Test for :trac:`15919`, :trac:`16244`::

            sage: Z = IntegerModRing(25) # random.sample, which was used pre #16244, has a threshold at 21!
            sage: Z[1]                   # since #8389, indexed access is used for ring extensions
            Traceback (most recent call last):
            ...
            ValueError: variable name '1' does not start with a letter
            sage: tester = InstanceTester(Z, elements=Z, max_runs=5)
            sage: list(tester.some_elements())
            [0, 1, 2, 3, 4]

            sage: C = cartesian_product([Z]*4)
            sage: len(C)
            390625
            sage: tester = InstanceTester(C, elements = C, max_runs=4)
            sage: list(tester.some_elements())
            [(0, 0, 0, 0), (0, 0, 0, 1), (0, 0, 0, 2), (0, 0, 0, 3)]
        """
        S = S or self._elements or self._instance.some_elements()
        from sage.misc.misc import some_tuples
        return list(some_tuples(S, repeat, self._max_runs, self._max_samples))


class PythonObjectWithTests(object):
    """
    Utility class for running basis tests on a plain Python object
    (that is not in SageObject). More test methods can be added here.

    EXAMPLES::

        sage: TestSuite("bla").run()
    """
    def __init__(self, instance):
        """
        EXAMPLES::

            sage: from sage.misc.sage_unittest import PythonObjectWithTests
            sage: x = PythonObjectWithTests(int(1)); x
            <sage.misc.sage_unittest.PythonObjectWithTests object at ...>
            sage: TestSuite(x).run()
        """
        self._instance = instance

    def _test_pickling(self, **options):
        """
        Checks that the instance in self can be pickled and unpickled properly.

        EXAMPLES::

            sage: from sage.misc.sage_unittest import PythonObjectWithTests
            sage: PythonObjectWithTests(int(1))._test_pickling()

        .. SEEALSO::

            :func:`dumps`, :func:`loads`
        """
        tester = instance_tester(self, **options)
        from sage.misc.persist import loads, dumps
        tester.assertEqual(loads(dumps(self._instance)), self._instance)

    def _test_new(self, **options):
        """
        Check that ``cls.__new__(cls)`` does not crash Python, with
        ``cls`` either the tested instance (if it's a type) or the type
        of the instance.

        It is perfectly legal for ``__new__`` to raise ordinary
        exceptions.

        EXAMPLES::

            sage: TestSuite(int(1)).run(verbose=True)
            running ._test_new() . . . pass
            running ._test_pickling() . . . pass
            sage: TestSuite(int).run(verbose=True)
            running ._test_new() . . . pass
            running ._test_pickling() . . . pass
        """
        cls = self._instance
        if not isinstance(cls, type):
            cls = type(cls)
        try:
            cls.__new__(cls)
        except Exception:
            pass



from .parent cimport Parent
from .coerce_dict cimport TripleDict

cpdef py_scalar_parent(py_type)
cpdef py_scalar_to_element(py)
cpdef bint parent_is_integers(P) except -1
cpdef bint is_numpy_type(t)
cpdef bint is_mpmath_type(t)


cdef class CoercionModel:
    # This MUST be a mapping to tuples, where each
    # tuple contains at least two elements that are either
    # None or of type Morphism.
    cdef readonly TripleDict _coercion_maps

    # This MUST be a mapping to actions.
    cdef readonly TripleDict _action_maps

    cpdef canonical_coercion(self, x, y)
    cpdef bin_op(self, x, y, op)
    cpdef richcmp(self, x, y, int op)

    cpdef coercion_maps(self, R, S)
    cpdef discover_coercion(self, R, S)
    cpdef verify_coercion_maps(self, R, S, homs, bint fix=*)
    cpdef verify_action(self, action, R, S, op, bint fix=*)

    cpdef get_action(self, R, S, op=*, r=*, s=*)
    cpdef discover_action(self, R, S, op, r=*, s=*)

    cdef bint _record_exceptions
    cpdef _record_exception(self)
    cdef readonly list _exception_stack
    cdef bint _exceptions_cleared

    cdef TripleDict _division_parents
    cpdef analyse(self, xp, yp, op=*)
    cpdef division_parent(self, Parent P)


# Unique global coercion_model instance
cdef CoercionModel coercion_model



# distutils: libraries = gmp

from .types cimport *

cdef extern from "gmp.h":

    ### Floating-point Functions ###

    # Initialization Functions
    void mpf_set_default_prec (unsigned long int prec)
    unsigned long int mpf_get_default_prec ()
    void mpf_init (mpf_t x)
    void mpf_init2 (mpf_t x, unsigned long int prec)
    void mpf_clear (mpf_t x)
    unsigned long int mpf_get_prec (mpf_t op)
    void mpf_set_prec (mpf_t rop, unsigned long int prec)
    void mpf_set_prec_raw (mpf_t rop, unsigned long int prec)

    # Assignment Functions
    void mpf_set (mpf_t rop, mpf_t op)
    void mpf_set_ui (mpf_t rop, unsigned long int op)
    void mpf_set_si (mpf_t rop, signed long int op)
    void mpf_set_d (mpf_t rop, double op)
    void mpf_set_z (mpf_t rop, mpz_t op)
    void mpf_set_q (mpf_t rop, mpq_t op)
    int mpf_set_str (mpf_t rop, char *str, int base)
    void mpf_swap (mpf_t rop1, mpf_t rop2)

    # Combined Initialization and Assignment Functions
    void mpf_init_set (mpf_t rop, mpf_t op)
    void mpf_init_set_ui (mpf_t rop, unsigned long int op)
    void mpf_init_set_si (mpf_t rop, signed long int op)
    void mpf_init_set_d (mpf_t rop, double op)
    int mpf_init_set_str (mpf_t rop, char *str, int base)

    # Conversion Functions
    double mpf_get_d (mpf_t op)
    double mpf_get_d_2exp (signed long int *exp, mpf_t op)
    long mpf_get_si (mpf_t op)
    unsigned long mpf_get_ui (mpf_t op)
    char * mpf_get_str (char *str, mp_exp_t *expptr, int base, size_t n_digits, mpf_t op)

    # Arithmetic Functions
    void mpf_add (mpf_t rop, mpf_t op1, mpf_t op2)
    void mpf_add_ui (mpf_t rop, mpf_t op1, unsigned long int op2)
    void mpf_sub (mpf_t rop, mpf_t op1, mpf_t op2)
    void mpf_ui_sub (mpf_t rop, unsigned long int op1, mpf_t op2)
    void mpf_sub_ui (mpf_t rop, mpf_t op1, unsigned long int op2)
    void mpf_mul (mpf_t rop, mpf_t op1, mpf_t op2)
    void mpf_mul_ui (mpf_t rop, mpf_t op1, unsigned long int op2)
    void mpf_div (mpf_t rop, mpf_t op1, mpf_t op2)
    void mpf_ui_div (mpf_t rop, unsigned long int op1, mpf_t op2)
    void mpf_div_ui (mpf_t rop, mpf_t op1, unsigned long int op2)
    void mpf_sqrt (mpf_t rop, mpf_t op)
    void mpf_sqrt_ui (mpf_t rop, unsigned long int op)
    void mpf_pow_ui (mpf_t rop, mpf_t op1, unsigned long int op2)
    void mpf_neg (mpf_t rop, mpf_t op)
    void mpf_abs (mpf_t rop, mpf_t op)
    void mpf_mul_2exp (mpf_t rop, mpf_t op1, unsigned long int op2)
    void mpf_div_2exp (mpf_t rop, mpf_t op1, unsigned long int op2)

    # Comparison Functions
    int mpf_cmp (mpf_t op1, mpf_t op2)
    int mpf_cmp_d (mpf_t op1, double op2)
    int mpf_cmp_ui (mpf_t op1, unsigned long int op2)
    int mpf_cmp_si (mpf_t op1, signed long int op2)
    int mpf_eq (mpf_t op1, mpf_t op2, unsigned long int op3)
    void mpf_reldiff (mpf_t rop, mpf_t op1, mpf_t op2)
    int mpf_sgn (mpf_t op)

    # Input and Output Functions
    # size_t mpf_out_str (file *stream, int base, size_t n_digits, mpf_t op)
    # size_t mpf_inp_str (mpf_t rop, file *stream, int base)

    # Miscellaneous Functions
    void mpf_ceil (mpf_t rop, mpf_t op)
    void mpf_floor (mpf_t rop, mpf_t op)
    void mpf_trunc (mpf_t rop, mpf_t op)
    bint mpf_integer_p (mpf_t op)
    bint mpf_fits_ulong_p (mpf_t op)
    bint mpf_fits_slong_p (mpf_t op)
    bint mpf_fits_uint_p (mpf_t op)
    bint mpf_fits_sint_p (mpf_t op)
    bint mpf_fits_ushort_p (mpf_t op)
    bint mpf_fits_sshort_p (mpf_t op)
    void mpf_urandomb (mpf_t rop, gmp_randstate_t state, unsigned long int nbits)
    void mpf_random2 (mpf_t rop, mp_size_t max_size, mp_exp_t exp)



r"""
Unicode Characters

This module provides Python identifiers for Unicode characters corresponding
to various mathematical symbols.

The syntax is ``unicode_XXX``, where ``XXX`` is the LaTeX name of the symbol,
stripped of any backslash or any curly brace (e.g. ``unicode_mathbbR`` for
``ℝ``).

EXAMPLES:

Operators::

    sage: from sage.typeset.unicode_characters import unicode_otimes
    sage: unicode_otimes
    '⊗'
    sage: hex(ord(_))
    '0x2297'
    sage: from sage.typeset.unicode_characters import unicode_bigotimes
    sage: unicode_bigotimes
    '⨂'
    sage: hex(ord(_))
    '0x2a02'
    sage: from sage.typeset.unicode_characters import unicode_wedge
    sage: unicode_wedge
    '∧'
    sage: hex(ord(_))
    '0x2227'
    sage: from sage.typeset.unicode_characters import unicode_bigwedge
    sage: unicode_bigwedge
    '⋀'
    sage: hex(ord(_))
    '0x22c0'
    sage: from sage.typeset.unicode_characters import unicode_partial
    sage: unicode_partial
    '∂'
    sage: hex(ord(_))
    '0x2202'

Arrows::

    sage: from sage.typeset.unicode_characters import unicode_to
    sage: unicode_to
    '→'
    sage: hex(ord(_))
    '0x2192'
    sage: from sage.typeset.unicode_characters import unicode_mapsto
    sage: unicode_mapsto
    '↦'
    sage: hex(ord(_))
    '0x21a6'

Letters::

    sage: from sage.typeset.unicode_characters import unicode_mathbbR
    sage: unicode_mathbbR
    'ℝ'
    sage: hex(ord(_))
    '0x211d'
    sage: from sage.typeset.unicode_characters import unicode_mathbbC
    sage: unicode_mathbbC
    'ℂ'
    sage: hex(ord(_))
    '0x2102'

"""

# ****************************************************************************
#       Copyright (C) 2021 Eric Gourgoulhon <eric.gourgoulhon@obspm.fr>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#                  https://www.gnu.org/licenses/
# ****************************************************************************

# Operators:

unicode_otimes = '\u2297'          # '⊗'

unicode_bigotimes = '\u2A02'       # '⨂'

unicode_wedge = '\u2227'           # '∧'

unicode_bigwedge = '\u22C0'        # '⋀'

unicode_partial = '\u2202'         # '∂'

# Arrows:

unicode_to = '\u2192'              # '→'

unicode_mapsto = '\u21A6'          # '↦'

# Letters:

unicode_mathbbR = '\u211D'         # 'ℝ'

unicode_mathbbC = '\u2102'         # 'ℂ'





r"""
The coercion model

The coercion model manages how elements of one parent get related to elements
of another. For example, the integer 2 can canonically be viewed as an element
of the rational numbers. (The parent of a non-element is its Python type.)

::

    sage: ZZ(2).parent()
    Integer Ring
    sage: QQ(2).parent()
    Rational Field

The most prominent role of the coercion model is to make sense of binary
operations between elements that have distinct parents. It does this by
finding a parent where both elements make sense, and doing the operation
there. For example::

    sage: a = 1/2; a.parent()
    Rational Field
    sage: b = ZZ['x'].gen(); b.parent()
    Univariate Polynomial Ring in x over Integer Ring
    sage: a+b
    x + 1/2
    sage: (a+b).parent()
    Univariate Polynomial Ring in x over Rational Field

If there is a coercion (see below) from one of the parents to the other,
the operation is always performed in the codomain of that coercion. Otherwise
a reasonable attempt to create a new parent with coercion maps from both
original parents is made. The results of these discoveries are cached.
On failure, a TypeError is always raised.

Some arithmetic operations (such as multiplication) can indicate an action
rather than arithmetic in a common parent. For example::

    sage: E = EllipticCurve('37a')
    sage: P = E(0,0)
    sage: 5*P
    (1/4 : -5/8 : 1)

where there is action of `\ZZ` on the points of `E` given by the additive
group law. Parents can specify how they act on or are acted upon by other
parents.

There are two kinds of ways to get from one parent to another, coercions and
conversions.

Coercions are canonical (possibly modulo a finite number of
deterministic choices) morphisms, and the set of all coercions between
all parents forms a commuting diagram (modulo possibly rounding
issues). `\ZZ \rightarrow \QQ` is an example of a
coercion. These are invoked implicitly by the coercion model.

Conversions try to construct an element out of their input if at all possible.
Examples include sections of coercions, creating an element from a string or
list, etc. and may fail on some inputs of a given type while succeeding on
others (i.e. they may not be defined on the whole domain). Conversions are
always explicitly invoked, and never used by the coercion model to resolve
binary operations.

For more information on how to specify coercions, conversions, and actions,
see the documentation for :class:`Parent`.
"""

#*****************************************************************************
#       Copyright (C) 2007 Robert Bradshaw <robertwb@math.washington.edu>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#                  http://www.gnu.org/licenses/
#*****************************************************************************

from cpython.object cimport (PyObject, PyTypeObject,
        PyObject_CallObject, PyObject_RichCompare, Py_TYPE,
        Py_EQ, Py_NE, Py_LT, Py_LE, Py_GT, Py_GE)
from cpython.weakref cimport PyWeakref_GET_OBJECT, PyWeakref_NewRef
from libc.string cimport strncmp
cimport gmpy2

cdef add, mul, truediv
from operator import add, mul, truediv

from .richcmp cimport rich_to_bool, revop
from .sage_object cimport SageObject
from .parent cimport Parent_richcmp_element_without_coercion
from .element cimport bin_op_exception, parent, Element
from .coerce_actions import LeftModuleAction, RightModuleAction
from .coerce_exceptions import CoercionException
from sage.rings.integer_fake cimport is_Integer
from sage.categories.map cimport Map
from sage.categories.morphism import IdentityMorphism
from sage.categories.action cimport Action, InverseAction, PrecomposedAction
from sage.sets.pythonclass cimport Set_PythonType

import traceback

from fractions import Fraction
cdef type FractionType = <type>Fraction

cpdef py_scalar_parent(py_type):
    """
    Returns the Sage equivalent of the given python type, if one exists.
    If there is no equivalent, return None.

    EXAMPLES::

        sage: from sage.structure.coerce import py_scalar_parent
        sage: py_scalar_parent(int)
        Integer Ring
        sage: py_scalar_parent(float)
        Real Double Field
        sage: py_scalar_parent(complex)
        Complex Double Field
        sage: py_scalar_parent(bool)
        Integer Ring
        sage: py_scalar_parent(dict),
        (None,)

        sage: import fractions
        sage: py_scalar_parent(fractions.Fraction)
        Rational Field

        sage: import numpy
        sage: py_scalar_parent(numpy.int16)
        Integer Ring
        sage: py_scalar_parent(numpy.int32)
        Integer Ring
        sage: py_scalar_parent(numpy.uint64)
        Integer Ring

        sage: py_scalar_parent(float)
        Real Double Field
        sage: py_scalar_parent(numpy.double)
        Real Double Field

        sage: py_scalar_parent(complex)
        Complex Double Field

        sage: import gmpy2
        sage: py_scalar_parent(gmpy2.mpz)
        Integer Ring
        sage: py_scalar_parent(gmpy2.mpq)
        Rational Field
        sage: py_scalar_parent(gmpy2.mpfr)
        Real Double Field
        sage: py_scalar_parent(gmpy2.mpc)
        Complex Double Field
    """
    if issubclass(py_type, int) or issubclass(py_type, long):
        import sage.rings.integer_ring
        return sage.rings.integer_ring.ZZ
    if py_type is FractionType:
        import sage.rings.rational_field
        return sage.rings.rational_field.QQ
    elif issubclass(py_type, float):
        import sage.rings.real_double
        return sage.rings.real_double.RDF
    elif issubclass(py_type, complex):
        import sage.rings.complex_double
        return sage.rings.complex_double.CDF
    elif is_numpy_type(py_type):
        import numpy
        if issubclass(py_type, numpy.integer):
            import sage.rings.integer_ring
            return sage.rings.integer_ring.ZZ
        elif issubclass(py_type, numpy.floating):
            import sage.rings.real_double
            return sage.rings.real_double.RDF
        elif issubclass(py_type, numpy.complexfloating):
            import sage.rings.complex_double
            return sage.rings.complex_double.CDF
        else:
            return None
    elif issubclass(py_type, gmpy2.mpz):
        import sage.rings.integer_ring
        return sage.rings.integer_ring.ZZ
    elif issubclass(py_type, gmpy2.mpq):
        import sage.rings.rational_field
        return sage.rings.rational_field.QQ
    elif issubclass(py_type, gmpy2.mpfr):
        import sage.rings.real_double
        return sage.rings.real_double.RDF
    elif issubclass(py_type, gmpy2.mpc):
        import sage.rings.complex_double
        return sage.rings.complex_double.CDF
    else:
        return None

cpdef py_scalar_to_element(x):
    """
    Convert ``x`` to a Sage :class:`~sage.structure.element.Element` if possible.

    If ``x`` was already an :class:`~sage.structure.element.Element` or if there is no obvious
    conversion possible, just return ``x`` itself.

    EXAMPLES::

        sage: from sage.structure.coerce import py_scalar_to_element
        sage: x = py_scalar_to_element(42)
        sage: x, parent(x)
        (42, Integer Ring)
        sage: x = py_scalar_to_element(int(42))
        sage: x, parent(x)
        (42, Integer Ring)
        sage: x = py_scalar_to_element(float(42))
        sage: x, parent(x)
        (42.0, Real Double Field)
        sage: x = py_scalar_to_element(complex(42))
        sage: x, parent(x)
        (42.0, Complex Double Field)
        sage: py_scalar_to_element('hello')
        'hello'

        sage: from fractions import Fraction
        sage: f = Fraction(int(2^100), int(3^100))
        sage: py_scalar_to_element(f)
        1267650600228229401496703205376/515377520732011331036461129765621272702107522001

    Note that bools are converted to 0 or 1::

        sage: py_scalar_to_element(False), py_scalar_to_element(True)
        (0, 1)

    Test gmpy2's types::

        sage: import gmpy2
        sage: x = py_scalar_to_element(gmpy2.mpz(42))
        sage: x, parent(x)
        (42, Integer Ring)
        sage: x = py_scalar_to_element(gmpy2.mpq('3/4'))
        sage: x, parent(x)
        (3/4, Rational Field)
        sage: x = py_scalar_to_element(gmpy2.mpfr(42.57))
        sage: x, parent(x)
        (42.57, Real Double Field)
        sage: x = py_scalar_to_element(gmpy2.mpc(int(42), int(42)))
        sage: x, parent(x)
        (42.0 + 42.0*I, Complex Double Field)

    Test compatibility with :func:`py_scalar_parent`::

        sage: from sage.structure.coerce import py_scalar_parent
        sage: elt = [True, int(42), float(42), complex(42)]
        sage: for x in elt:
        ....:     assert py_scalar_parent(type(x)) == py_scalar_to_element(x).parent()

        sage: import numpy
        sage: elt = [numpy.int8('-12'),  numpy.uint8('143'),
        ....:        numpy.int16('-33'), numpy.uint16('122'),
        ....:        numpy.int32('-19'), numpy.uint32('44'),
        ....:        numpy.int64('-3'),  numpy.uint64('552'),
        ....:        numpy.float16('-1.23'), numpy.float32('-2.22'),
        ....:        numpy.float64('-3.412'), numpy.complex64(1.2+I),
        ....:         numpy.complex128(-2+I)]
        sage: for x in elt:
        ....:     assert py_scalar_parent(type(x)) == py_scalar_to_element(x).parent()

        sage: elt = [gmpy2.mpz(42), gmpy2.mpq('3/4'),
        ....:        gmpy2.mpfr(42.57), gmpy2.mpc(int(42), int(42))]
        sage: for x in elt:
        ....:     assert py_scalar_parent(type(x)) == py_scalar_to_element(x).parent()
    """
    if isinstance(x, Element):
        return x
    elif isinstance(x, (int, long)):
        from sage.rings.integer import Integer
        return Integer(x)
    elif type(x) is FractionType:
        from sage.rings.rational import Rational
        return Rational(x)
    elif isinstance(x, float):
        from sage.rings.real_double import RDF
        return RDF(x)
    elif isinstance(x, complex):
        from sage.rings.complex_double import CDF
        return CDF(x)
    elif is_numpy_type(type(x)):
        import numpy
        if isinstance(x, numpy.integer):
            from sage.rings.integer import Integer
            return Integer(x)
        elif isinstance(x, numpy.floating):
            from sage.rings.real_double import RDF
            return RDF(x)
        elif isinstance(x, numpy.complexfloating):
            from sage.rings.complex_double import CDF
            return CDF(x)
        else:
            return x
    elif type(x) is gmpy2.mpz:
            from sage.rings.integer import Integer
            return Integer(x)
    elif type(x) is gmpy2.mpq:
        from sage.rings.rational import Rational
        return Rational(x)
    elif type(x) is gmpy2.mpfr:
        from sage.rings.real_double import RDF
        return RDF(x)
    elif type(x) is gmpy2.mpc:
        from sage.rings.complex_double import CDF
        return CDF(x)
    else:
        return x


cpdef bint parent_is_integers(P) except -1:
    """
    Check whether the type or parent represents the ring of integers.

    EXAMPLES::

        sage: from sage.structure.coerce import parent_is_integers
        sage: parent_is_integers(int)
        True
        sage: parent_is_integers(float)
        False
        sage: parent_is_integers(bool)
        True
        sage: parent_is_integers(dict)
        False

        sage: import numpy
        sage: parent_is_integers(numpy.int16)
        True
        sage: parent_is_integers(numpy.uint64)
        True
        sage: parent_is_integers(float)
        False

        sage: import gmpy2
        sage: parent_is_integers(gmpy2.mpz)
        True
        sage: parent_is_integers(gmpy2.mpq)
        False

    Ensure (:trac:`27893`) is fixed::

        sage: K.<f> = QQ[]
        sage: gmpy2.mpz(2) * f
        2*f
    """
    if isinstance(P, type):
        if issubclass(P, int) or issubclass(P, long):
            return True
        elif is_numpy_type(P):
            from numpy import integer
            return issubclass(P, integer)
        elif issubclass(P, gmpy2.mpz):
            return True
        else:
            return False
    else:
        from sage.rings.integer_ring import ZZ
        return P is ZZ

def parent_is_numerical(P):
    r"""
    Test if elements of the parent or type ``P`` can be numerically evaluated
    as complex numbers (in a canonical way).

    EXAMPLES::

        sage: from sage.structure.coerce import parent_is_numerical
        sage: import gmpy2, numpy
        sage: [parent_is_numerical(R) for R in [RR, CC, QQ, QuadraticField(-1),
        ....:         int, complex, gmpy2.mpc, numpy.complexfloating]]
        [True, True, True, True, True, True, True, True]
        sage: [parent_is_numerical(R) for R in [SR, QQ['x'], QQ[['x']], str]]
        [False, False, False, False]
        sage: [parent_is_numerical(R) for R in [RIF, RBF, CIF, CBF]]
        [False, False, False, False]
    """
    if not isinstance(P, Parent):
        P = py_scalar_parent(P)
        if P is None:
            return False
    return P._is_numerical()

def parent_is_real_numerical(P):
    r"""
    Test if elements of the parent or type ``P`` can be numerically evaluated
    as real numbers (in a canonical way).

    EXAMPLES::

        sage: from sage.structure.coerce import parent_is_real_numerical
        sage: import gmpy2, numpy
        sage: [parent_is_real_numerical(R) for R in [RR, QQ, ZZ, RLF,
        ....:         QuadraticField(2), int, float, gmpy2.mpq, numpy.integer]]
        [True, True, True, True, True, True, True, True, True]
        sage: [parent_is_real_numerical(R) for R in [CC, QuadraticField(-1),
        ....:         complex, gmpy2.mpc, numpy.complexfloating]]
        [False, False, False, False, False]
        sage: [parent_is_real_numerical(R) for R in [SR, QQ['x'], QQ[['x']], str]]
        [False, False, False, False]
        sage: [parent_is_real_numerical(R) for R in [RIF, RBF, CIF, CBF]]
        [False, False, False, False]
    """
    if not isinstance(P, Parent):
        P = py_scalar_parent(P)
        if P is None:
            return False
    return P._is_real_numerical()


cpdef bint is_numpy_type(t):
    """
    Return ``True`` if and only if `t` is a type whose name starts
    with ``numpy.``

    EXAMPLES::

        sage: from sage.structure.coerce import is_numpy_type
        sage: import numpy
        sage: is_numpy_type(numpy.int16)
        True
        sage: is_numpy_type(numpy.floating)
        True
        sage: is_numpy_type(numpy.ndarray)
        True
        sage: is_numpy_type(numpy.matrix)
        True
        sage: is_numpy_type(int)
        False
        sage: is_numpy_type(Integer)
        False
        sage: is_numpy_type(Sudoku)
        False
        sage: is_numpy_type(None)
        False

    TESTS:

    This used to crash Sage (:trac:`20715`)::

        sage: is_numpy_type(object)
        False
        sage: 1 + object()
        Traceback (most recent call last):
        ...
        TypeError: unsupported operand parent(s) for +: 'Integer Ring' and
        '<class 'object'>'
    """
    if not isinstance(t, type):
        return False
    cdef PyTypeObject* T = <PyTypeObject*>t
    if strncmp(T.tp_name, "numpy.", 6) == 0:
        return True
    # Check base type. This is needed to detect numpy.matrix.
    if T.tp_base != NULL and strncmp(T.tp_base.tp_name, "numpy.", 6) == 0:
        return True
    return False

cpdef bint is_mpmath_type(t):
    r"""
    Check whether the type ``t`` is a type whose name starts with either
    ``mpmath.`` or ``sage.libs.mpmath.``.

    EXAMPLES::

        sage: from sage.structure.coerce import is_mpmath_type
        sage: is_mpmath_type(int)
        False
        sage: import mpmath
        sage: is_mpmath_type(mpmath.mpc(2))
        False
        sage: is_mpmath_type(type(mpmath.mpc(2)))
        True
        sage: is_mpmath_type(type(mpmath.mpf(2)))
        True
    """
    return isinstance(t, type) and \
           strncmp((<PyTypeObject*>t).tp_name, "sage.libs.mpmath.", 17) == 0


cdef class CoercionModel:
    """
    See also sage.categories.pushout

    EXAMPLES::

        sage: f = ZZ['t','x'].0 + QQ['x'].0 + CyclotomicField(13).gen(); f
        t + x + zeta13
        sage: f.parent()
        Multivariate Polynomial Ring in t, x over Cyclotomic Field of order 13 and degree 12
        sage: ZZ['x','y'].0 + ~Frac(QQ['y']).0
        (x*y + 1)/y
        sage: MatrixSpace(ZZ['x'], 2, 2)(2) + ~Frac(QQ['x']).0
        [(2*x + 1)/x           0]
        [          0 (2*x + 1)/x]
        sage: f = ZZ['x,y,z'].0 + QQ['w,x,z,a'].0; f
        w + x
        sage: f.parent()
        Multivariate Polynomial Ring in w, x, y, z, a over Rational Field
        sage: ZZ['x,y,z'].0 + ZZ['w,x,z,a'].1
        2*x

    TESTS:

    Check that :trac:`8426` is fixed (see also :trac:`18076`)::

        sage: import numpy
        sage: x = polygen(RR)
        sage: numpy.float32('1.5') * x
        1.50000000000000*x
        sage: x * numpy.float32('1.5')
        1.50000000000000*x
        sage: p = x**3 + 2*x - 1
        sage: p(float('1.2'))
        3.12800000000000
        sage: p(int('2'))
        11.0000000000000

    This used to fail (see :trac:`18076`)::

        sage: 1/3 + numpy.int8('12')
        37/3
        sage: -2/3 + numpy.int16('-2')
        -8/3
        sage: 2/5 + numpy.uint8('2')
        12/5

    The numpy types do not interact well with the Sage coercion framework. More
    precisely, if a numpy type is the first operand in a binary operation then
    this operation is done in numpy. The result is hence a numpy type::

        sage: numpy.uint8('2') + 3
        5
        sage: type(_)
        <class 'numpy.int32'>  # 32-bit
        <class 'numpy.int64'>  # 64-bit

        sage: numpy.int8('12') + 1/3
        12.333333333333334
        sage: type(_)
        <class 'numpy.float64'>

    AUTHOR:

    - Robert Bradshaw
    """
    def __init__(self):
        """
        EXAMPLES::

            sage: from sage.structure.coerce import CoercionModel
            sage: cm = CoercionModel()
            sage: K = NumberField(x^2-2, 'a')
            sage: A = cm.get_action(ZZ, K, operator.mul)
            sage: f, g = cm.coercion_maps(QQ, int)
            sage: f, g = cm.coercion_maps(ZZ, int)
        """
        self.reset_cache()

    def reset_cache(self):
        """
        Clear the coercion cache.

        This should have no impact on the result of arithmetic operations, as
        the exact same coercions and actions will be re-discovered when needed.

        It may be useful for debugging, and may also free some memory.

        EXAMPLES::

            sage: cm = sage.structure.element.get_coercion_model()
            sage: len(cm.get_cache()[0])    # random
            42
            sage: cm.reset_cache()
            sage: cm.get_cache()
            ({}, {})
        """
        # This MUST be a mapping of tuples, where each
        # tuple contains at least two elements that are either
        # None or of type Map.
        self._coercion_maps = TripleDict()
        # This MUST be a mapping to actions.
        self._action_maps = TripleDict()
        # This is a mapping from Parents to Parents, storing the result of division in the given parent.
        self._division_parents = TripleDict()

    def get_cache(self):
        """
        This returns the current cache of coercion maps and actions, primarily
        useful for debugging and introspection.

        EXAMPLES::

            sage: cm = sage.structure.element.get_coercion_model()
            sage: cm.canonical_coercion(1,2/3)
            (1, 2/3)
            sage: maps, actions = cm.get_cache()

        Now let us see what happens when we do a binary operations with
        an integer and a rational::

            sage: left_morphism_ref, right_morphism_ref = maps[ZZ, QQ]

        Note that by :trac:`14058` the coercion model only stores a weak
        reference to the coercion maps in this case::

            sage: left_morphism_ref
            <weakref at ...; to 'sage.rings.rational.Z_to_Q' at ...>

        Moreover, the weakly referenced coercion map uses only a weak
        reference to the codomain::

            sage: left_morphism_ref()
            (map internal to coercion system -- copy before use)
            Natural morphism:
              From: Integer Ring
              To:   Rational Field

        To get an actual valid map, we simply copy the weakly referenced
        coercion map::

            sage: print(copy(left_morphism_ref()))
            Natural morphism:
              From: Integer Ring
              To:   Rational Field
            sage: print(right_morphism_ref)
            None

        We can see that it coerces the left operand from an integer to a
        rational, and doesn't do anything to the right.

        Now for some actions::

            sage: R.<x> = ZZ['x']
            sage: 1/2 * x
            1/2*x
            sage: maps, actions = cm.get_cache()
            sage: act = actions[QQ, R, operator.mul]; act
            Left scalar multiplication by Rational Field on Univariate Polynomial Ring in x over Integer Ring
            sage: act.actor()
            Rational Field
            sage: act.domain()
            Univariate Polynomial Ring in x over Integer Ring
            sage: act.codomain()
            Univariate Polynomial Ring in x over Rational Field
            sage: act(1/5, x+10)
            1/5*x + 2
        """
        d1 = {(S, R): mors for (S, R, op), mors in self._coercion_maps.items()}
        d2 = self._action_maps.copy()
        return d1, d2

    def record_exceptions(self, bint value=True):
        r"""
        Enables (or disables) recording of the exceptions suppressed during
        arithmetic.

        Each time that record_exceptions is called (either enabling or disabling
        the record), the exception_stack is cleared.

        TESTS::

            sage: cm = sage.structure.element.get_coercion_model()
            sage: cm.record_exceptions()
            sage: cm._test_exception_stack()
            sage: cm.exception_stack()
            ['Traceback (most recent call last):\n  File "sage/structure/coerce.pyx", line ...TypeError: just a test']
            sage: cm.record_exceptions(False)
            sage: cm._test_exception_stack()
            sage: cm.exception_stack()
            []
        """
        self._record_exceptions = value
        self._exceptions_cleared = True
        self._exception_stack = []

    cpdef _record_exception(self):
        r"""
        Pushes the last exception that occurred onto the stack for later reference,
        for internal use.

        If the stack has not yet been flagged as cleared, we clear it now (rather
        than wasting time to do so for successful operations).

        TESTS::

            sage: cm = sage.structure.element.get_coercion_model()
            sage: cm.record_exceptions()
            sage: 1+1/2+2 # make sure there aren't any errors hanging around
            7/2
            sage: cm.exception_stack()
            []
            sage: cm._test_exception_stack()
            sage: cm.exception_stack()
            ['Traceback (most recent call last):\n  File "sage/structure/coerce.pyx", line ...TypeError: just a test']

        The function _test_exception_stack is executing the following code::

            try:
                raise TypeError("just a test")
            except TypeError:
                cm._record_exception()
        """
        if not self._record_exceptions:
            return
        if not self._exceptions_cleared:
            self._exception_stack = []
            self._exceptions_cleared = True
        self._exception_stack.append(traceback.format_exc().strip())

    def _test_exception_stack(self):
        r"""
        A function to test the exception stack.

        EXAMPLES::

            sage: cm = sage.structure.element.get_coercion_model()
            sage: cm.record_exceptions()
            sage: 1 + 1/11 # make sure there aren't any errors hanging around
            12/11
            sage: cm.exception_stack()
            []
            sage: cm._test_exception_stack()
            sage: cm.exception_stack()
            ['Traceback (most recent call last):\n  File "sage/structure/coerce.pyx", line ...TypeError: just a test']
        """
        try:
            raise TypeError("just a test")
        except TypeError:
            self._record_exception()

    def exception_stack(self):
        r"""
        Returns the list of exceptions that were caught in the course of
        executing the last binary operation. Useful for diagnosis when
        user-defined maps or actions raise exceptions that are caught in
        the course of coercion detection.

        If all went well, this should be the empty list. If things aren't
        happening as you expect, this is a good place to check. See also
        :func:`coercion_traceback`.

        EXAMPLES::

            sage: cm = sage.structure.element.get_coercion_model()
            sage: cm.record_exceptions()
            sage: 1/2 + 2
            5/2
            sage: cm.exception_stack()
            []
            sage: 1/2 + GF(3)(2)
            Traceback (most recent call last):
            ...
            TypeError: unsupported operand parent(s) for +: 'Rational Field' and 'Finite Field of size 3'

        Now see what the actual problem was::

            sage: import traceback
            sage: cm.exception_stack()
            ['Traceback (most recent call last):...', 'Traceback (most recent call last):...']
            sage: print(cm.exception_stack()[-1])
            Traceback (most recent call last):
            ...
            TypeError: no common canonical parent for objects with parents: 'Rational Field' and 'Finite Field of size 3'

        This is typically accessed via the :func:`coercion_traceback` function.

        ::

            sage: coercion_traceback()
            Traceback (most recent call last):
            ...
            TypeError: no common canonical parent for objects with parents: 'Rational Field' and 'Finite Field of size 3'
        """
        if not self._exceptions_cleared:
            self._exception_stack = []
            self._exceptions_cleared = True
        return self._exception_stack


    def explain(self, xp, yp, op=mul, int verbosity=2):
        """
        This function can be used to understand what coercions will happen
        for an arithmetic operation between xp and yp (which may be either
        elements or parents). If the parent of the result can be determined
        then it will be returned.

        EXAMPLES::

            sage: cm = sage.structure.element.get_coercion_model()

            sage: cm.explain(ZZ, ZZ)
            Identical parents, arithmetic performed immediately.
            Result lives in Integer Ring
            Integer Ring

            sage: cm.explain(QQ, int)
            Coercion on right operand via
                Native morphism:
                  From: Set of Python objects of class 'int'
                  To:   Rational Field
            Arithmetic performed after coercions.
            Result lives in Rational Field
            Rational Field

            sage: R = ZZ['x']
            sage: cm.explain(R, QQ)
            Action discovered.
                Right scalar multiplication by Rational Field on Univariate Polynomial Ring in x over Integer Ring
            Result lives in Univariate Polynomial Ring in x over Rational Field
            Univariate Polynomial Ring in x over Rational Field

            sage: cm.explain(ZZ['x'], QQ, operator.add)
            Coercion on left operand via
                Ring morphism:
                  From: Univariate Polynomial Ring in x over Integer Ring
                  To:   Univariate Polynomial Ring in x over Rational Field
                  Defn: Induced from base ring by
                        Natural morphism:
                          From: Integer Ring
                          To:   Rational Field
            Coercion on right operand via
                Polynomial base injection morphism:
                  From: Rational Field
                  To:   Univariate Polynomial Ring in x over Rational Field
            Arithmetic performed after coercions.
            Result lives in Univariate Polynomial Ring in x over Rational Field
            Univariate Polynomial Ring in x over Rational Field

        Sometimes with non-sage types there is not enough information to deduce
        what will actually happen::

            sage: R100 = RealField(100)
            sage: cm.explain(R100, float, operator.add)
            Right operand is numeric, will attempt coercion in both directions.
            Unknown result parent.
            sage: parent(R100(1) + float(1))
            <class 'float'>
            sage: cm.explain(QQ, float, operator.add)
            Right operand is numeric, will attempt coercion in both directions.
            Unknown result parent.
            sage: parent(QQ(1) + float(1))
            <class 'float'>

        Special care is taken to deal with division::

            sage: cm.explain(ZZ, ZZ, operator.truediv)
            Identical parents, arithmetic performed immediately.
            Result lives in Rational Field
            Rational Field

            sage: ZZx = ZZ['x']
            sage: QQx = QQ['x']
            sage: cm.explain(ZZx, QQx, operator.truediv)
            Coercion on left operand via
                Ring morphism:
                  From: Univariate Polynomial Ring in x over Integer Ring
                  To:   Univariate Polynomial Ring in x over Rational Field
                  Defn: Induced from base ring by
                        Natural morphism:
                          From: Integer Ring
                          To:   Rational Field
            Arithmetic performed after coercions.
            Result lives in Fraction Field of Univariate Polynomial Ring in x over Rational Field
            Fraction Field of Univariate Polynomial Ring in x over Rational Field

            sage: cm.explain(int, ZZ, operator.truediv)
            Coercion on left operand via
                Native morphism:
                  From: Set of Python objects of class 'int'
                  To:   Integer Ring
            Arithmetic performed after coercions.
            Result lives in Rational Field
            Rational Field

            sage: cm.explain(ZZx, ZZ, operator.truediv)
            Action discovered.
                Right inverse action by Rational Field on Univariate Polynomial Ring in x over Integer Ring
                with precomposition on right by Natural morphism:
                  From: Integer Ring
                  To:   Rational Field
            Result lives in Univariate Polynomial Ring in x over Rational Field
            Univariate Polynomial Ring in x over Rational Field

        .. NOTE::

           This function is accurate only in so far as :meth:`analyse` is kept
           in sync with the :meth:`bin_op` and
           :meth:`canonical_coercion` which are kept separate for
           maximal efficiency.
        """
        all, res = self.analyse(xp, yp, op)
        indent = " " * 4
        if verbosity >= 2:
            print("\n".join(s if isinstance(s, str)
                            else (indent + repr(s).replace("\n", "\n" + indent))
                            for s in all))
        elif verbosity >= 1:
            print("\n".join(s for s in all if isinstance(s, str)))
        if verbosity >= 1:
            if res is None:
                print("Unknown result parent.")
            else:
                print("Result lives in {}".format(res))
        return res

    cpdef analyse(self, xp, yp, op=mul):
        """
        Emulate the process of doing arithmetic between xp and yp, returning
        a list of steps and the parent that the result will live in. The
        ``explain`` function is easier to use, but if one wants access to
        the actual morphism and action objects (rather than their string
        representations) then this is the function to use.

        EXAMPLES::

            sage: cm = sage.structure.element.get_coercion_model()
            sage: GF7 = GF(7)
            sage: steps, res = cm.analyse(GF7, ZZ)
            sage: steps
            ['Coercion on right operand via', Natural morphism:
              From: Integer Ring
              To:   Finite Field of size 7, 'Arithmetic performed after coercions.']
            sage: res
            Finite Field of size 7
            sage: f = steps[1]; type(f)
            <class 'sage.rings.finite_rings.integer_mod.Integer_to_IntegerMod'>
            sage: f(100)
            2
        """
        self._exceptions_cleared = False
        res = None
        if not isinstance(xp, type) and not isinstance(xp, Parent):
            xp = parent(xp)
        if not isinstance(yp, type) and not isinstance(yp, Parent):
            yp = parent(yp)

        all = []
        if xp is yp:
            all.append("Identical parents, arithmetic performed immediately." % xp)
            if op is truediv and isinstance(xp, Parent):
                xp = self.division_parent(xp)
            return all, xp
        if xp == yp:
            all.append("Equal but distinct parents.")

        action = self.get_action(xp, yp, op)
        if action is not None:
            all.append("Action discovered.")
            all.append(action)
            return all, action.codomain()

        homs = self.discover_coercion(xp, yp)
        if homs is not None:
            x_mor, y_mor = homs
            if x_mor is not None:
                x_mor = x_mor.__copy__()
                all.append("Coercion on left operand via")
                all.append(x_mor)
                res = x_mor.codomain()
            if y_mor is not None:
                y_mor = y_mor.__copy__()
                all.append("Coercion on right operand via")
                all.append(y_mor)
                if res is not None and res is not y_mor.codomain():
                    raise RuntimeError("BUG in coercion model: codomains not equal!", x_mor, y_mor)
                res = y_mor.codomain()
            all.append("Arithmetic performed after coercions.")
            if op is truediv and isinstance(res, Parent):
                res = self.division_parent(res)
            return all, res

        if isinstance(yp, Parent) and xp in [int, long, float, complex, bool]:
            mor = yp._internal_coerce_map_from(xp)
            if mor is not None:
                mor = mor.__copy__()
                all.append("Coercion on numeric left operand via")
                all.append(mor)
                if op is truediv and isinstance(yp, Parent):
                    yp = self.division_parent(yp)
                return all, yp
            all.append("Left operand is numeric, will attempt coercion in both directions.")
        elif type(xp) is type:
            all.append("Left operand is not Sage element, will try _sage_.")

        if isinstance(xp, Parent) and yp in [int, long, float, complex, bool]:
            mor = xp._internal_coerce_map_from(yp)
            if mor is not None:
                mor = mor.__copy__()
                all.append("Coercion on numeric right operand via")
                all.append(mor)
                if op is truediv and isinstance(xp, Parent):
                    xp = self.division_parent(xp)
                return all, xp
            all.append("Right operand is numeric, will attempt coercion in both directions.")
        elif type(yp) is type:
            all.append("Right operand is not Sage element, will try _sage_.")

        return all, None

    def common_parent(self, *args):
        """
        Computes a common parent for all the inputs. It's essentially
        an `n`-ary canonical coercion except it can operate on parents
        rather than just elements.

        INPUT:

        - ``args`` -- a set of elements and/or parents

        OUTPUT:

        A :class:`Parent` into which each input should coerce, or raises a
        ``TypeError`` if no such :class:`Parent` can be found.

        EXAMPLES::

            sage: cm = sage.structure.element.get_coercion_model()
            sage: cm.common_parent(ZZ, QQ)
            Rational Field
            sage: cm.common_parent(ZZ, QQ, RR)
            Real Field with 53 bits of precision
            sage: ZZT = ZZ[['T']]
            sage: QQT = QQ['T']
            sage: cm.common_parent(ZZT, QQT, RDF)
            Power Series Ring in T over Real Double Field
            sage: cm.common_parent(4r, 5r)
            <class 'int'>
            sage: cm.common_parent(int, float, ZZ)
            <class 'float'>
            sage: real_fields = [RealField(prec) for prec in [10,20..100]]
            sage: cm.common_parent(*real_fields)
            Real Field with 10 bits of precision

        There are some cases where the ordering does matter, but if a parent
        can be found it is always the same::

            sage: QQxy = QQ['x,y']
            sage: QQyz = QQ['y,z']
            sage: cm.common_parent(QQxy, QQyz) == cm.common_parent(QQyz, QQxy)
            True
            sage: QQzt = QQ['z,t']
            sage: cm.common_parent(QQxy, QQyz, QQzt)
            Multivariate Polynomial Ring in x, y, z, t over Rational Field
            sage: cm.common_parent(QQxy, QQzt, QQyz)
            Traceback (most recent call last):
            ...
            TypeError: no common canonical parent for objects with parents: 'Multivariate Polynomial Ring in x, y over Rational Field' and 'Multivariate Polynomial Ring in z, t over Rational Field'
        """
        base = None
        for x in args:
            if not isinstance(x, Parent) and not isinstance(x, type):
               x = parent(x)
            if base is None:
                base = x
            if isinstance(base, Parent) and (<Parent>base).has_coerce_map_from(x):
                continue
            elif isinstance(x, Parent) and (<Parent>x).has_coerce_map_from(base):
                base = x
            else:
                a = base.an_element() if isinstance(base, Parent) else base(1)
                b = x.an_element() if isinstance(x, Parent) else x(1)
                base = parent(self.canonical_coercion(a, b)[0])
        return base

    cpdef division_parent(self, Parent P):
        r"""
        Deduces where the result of division in ``P`` lies by
        calculating the inverse of ``P.one()`` or ``P.an_element()``.

        The result is cached.

        EXAMPLES::

            sage: cm = sage.structure.element.get_coercion_model()
            sage: cm.division_parent(ZZ)
            Rational Field
            sage: cm.division_parent(QQ)
            Rational Field
            sage: ZZx = ZZ['x']
            sage: cm.division_parent(ZZx)
            Fraction Field of Univariate Polynomial Ring in x over Integer Ring
            sage: K = GF(41)
            sage: cm.division_parent(K)
            Finite Field of size 41
            sage: Zmod100 = Integers(100)
            sage: cm.division_parent(Zmod100)
            Ring of integers modulo 100
            sage: S5 = SymmetricGroup(5)                                                # optional - sage.groups
            sage: cm.division_parent(S5)                                                # optional - sage.groups
            Symmetric group of order 5! as a permutation group
        """
        try:
            return self._division_parents.get(P, None, None)
        except KeyError:
            pass
        try:
            ret = parent(~P.one())
        except Exception:
            self._record_exception()
            ret = parent(~P.an_element())
        self._division_parents.set(P, None, None, ret)
        return ret

    cpdef bin_op(self, x, y, op):
        """
        Execute the operation op on x and y. It first looks for an action
        corresponding to op, and failing that, it tries to coerces x and y
        into a common parent and calls op on them.

        If it cannot make sense of the operation, a TypeError is raised.

        INPUT:

        - ``x``  - the left operand

        - ``y``  - the right operand

        - ``op`` - a python function taking 2 arguments

          .. NOTE::

             op is often an arithmetic operation, but need not be so.

        EXAMPLES::

            sage: cm = sage.structure.element.get_coercion_model()
            sage: cm.bin_op(1/2, 5, operator.mul)
            5/2

        The operator can be any callable::

            sage: R.<x> = ZZ['x']
            sage: cm.bin_op(x^2-1, x+1, gcd)
            x + 1

        Actions are detected and performed::

            sage: M = matrix(ZZ, 2, 2, range(4))
            sage: V = vector(ZZ, [5,7])
            sage: cm.bin_op(M, V, operator.mul)
            (7, 31)

        TESTS::

            sage: class Foo(object):
            ....:     def __rmul__(self, left):
            ....:         return 'hello'
            sage: H = Foo()
            sage: print(int(3)*H)
            hello
            sage: print(Integer(3)*H)
            hello
            sage: print(H*3)
            Traceback (most recent call last):
            ...
            TypeError: unsupported operand parent(s) for *: '<class '__main__.Foo'>' and 'Integer Ring'

            sage: class Nonsense(object):
            ....:     def __init__(self, s):
            ....:         self.s = s
            ....:     def __repr__(self):
            ....:         return self.s
            ....:     def __mul__(self, x):
            ....:         return Nonsense(self.s + chr(x%256))
            ....:     __add__ = __mul__
            ....:     def __rmul__(self, x):
            ....:         return Nonsense(chr(x%256) + self.s)
            ....:     __radd__ = __rmul__
            sage: a = Nonsense('blahblah')
            sage: a*80
            blahblahP
            sage: 80*a
            Pblahblah
            sage: a+80
            blahblahP
            sage: 80+a
            Pblahblah
        """
        self._exceptions_cleared = False

        # If parents are equal, we can just call op()
        xp = parent(x)
        yp = parent(y)
        if xp is yp:
            return op(x,y)

        # Actions take preference over common-parent coercions
        try:
            action = self._action_maps.get(xp, yp, op)
        except KeyError:
            action = self.get_action(xp, yp, op, x, y)
        if action is not None:
            if (<Action>action)._is_left:
                return (<Action>action)._act_(x, y)
            else:
                return (<Action>action)._act_(y, x)

        # Now coerce to a common parent and do the operation there
        try:
            xy = self.canonical_coercion(x, y)
        except TypeError:
            self._record_exception()
        else:
            return PyObject_CallObject(op, xy)

        if op is mul:
            # elements may also act on non-elements
            # (e.g. sequences or parents)
            if not isinstance(y, Element) or not isinstance(x, Element):
                try:
                    if hasattr(x, '_act_on_'):
                        res = x._act_on_(y, True)
                        if res is not None: return res
                except CoercionException:
                    self._record_exception()

                try:
                    if hasattr(x, '_acted_upon_'):
                        res = x._acted_upon_(y, True)
                        if res is not None: return res
                except CoercionException:
                    self._record_exception()

                try:
                    if hasattr(y, '_act_on_'):
                        res = y._act_on_(x, False)
                        if res is not None: return res
                except CoercionException:
                    self._record_exception()

                try:
                    if hasattr(y, '_acted_upon_'):
                        res = y._acted_upon_(x, False)
                        if res is not None: return res
                except CoercionException:
                    self._record_exception()

        if not isinstance(y, Element):
            op_name = op.__name__
            mul_method = getattr(y, '__r%s__'%op_name, None)
            if mul_method is not None:
                res = mul_method(x)
                if res is not None and res is not NotImplemented:
                    return res

        # We should really include the underlying error.
        # This causes so much headache.
        raise bin_op_exception(op, x, y)

    cpdef canonical_coercion(self, x, y):
        r"""
        Given two elements x and y, with parents S and R respectively,
        find a common parent Z such that there are coercions
        `f: S \mapsto Z` and `g: R \mapsto Z` and return `f(x), g(y)`
        which will have the same parent.

        Raises a type error if no such Z can be found.

        EXAMPLES::

            sage: cm = sage.structure.element.get_coercion_model()
            sage: cm.canonical_coercion(mod(2, 10), 17)
            (2, 7)
            sage: x, y = cm.canonical_coercion(1/2, matrix(ZZ, 2, 2, range(4)))
            sage: x
            [1/2   0]
            [  0 1/2]
            sage: y
            [0 1]
            [2 3]
            sage: parent(x) is parent(y)
            True

        There is some support for non-Sage datatypes as well::

            sage: x, y = cm.canonical_coercion(int(5), 10)
            sage: type(x), type(y)
            (<class 'sage.rings.integer.Integer'>, <class 'sage.rings.integer.Integer'>)


            sage: x, y = cm.canonical_coercion(int(5), complex(3))
            sage: type(x), type(y)
            (<class 'complex'>, <class 'complex'>)

            sage: class MyClass:
            ....:     def _sage_(self):
            ....:         return 13
            sage: a, b = cm.canonical_coercion(MyClass(), 1/3)
            sage: a, b
            (13, 1/3)
            sage: type(a)
            <class 'sage.rings.rational.Rational'>

        We also make an exception for 0, even if $\ZZ$ does not map in::

            sage: canonical_coercion(vector([1, 2, 3]), 0)
            ((1, 2, 3), (0, 0, 0))
            sage: canonical_coercion(GF(5)(0), float(0))
            (0, 0)
        """
        xp = parent(x)
        yp = parent(y)
        if xp is yp:
            return x,y

        cdef Element x_elt, y_elt
        coercions = self.coercion_maps(xp, yp)
        if coercions is not None:
            x_map, y_map = coercions
            if x_map is not None:
                x_elt = (<Map>x_map)._call_(x)
            else:
                x_elt = x
            if y_map is not None:
                y_elt = (<Map>y_map)._call_(y)
            else:
                y_elt = y
            if x_elt is None:
                raise RuntimeError("BUG in map, returned None %s %s %s" % (x, type(x_map), x_map))
            elif y_elt is None:
                raise RuntimeError("BUG in map, returned None %s %s %s" % (y, type(y_map), y_map))
            if x_elt._parent is y_elt._parent:
                # We must verify this as otherwise we are prone to
                # getting into an infinite loop in c, and the above
                # maps may be written by (imperfect) users.
                return x_elt,y_elt
            elif x_elt._parent == y_elt._parent:
                # TODO: Non-uniqueness of parents strikes again!
                y_elt = parent(x_elt)(y_elt)
                if x_elt._parent is y_elt._parent:
                    return x_elt,y_elt
            self._coercion_error(x, x_map, x_elt, y, y_map, y_elt)

        cdef bint x_numeric = isinstance(x, (int, long, float, complex))
        cdef bint y_numeric = isinstance(y, (int, long, float, complex))

        if not x_numeric and is_numpy_type(type(x)):
            import numpy
            x_numeric = isinstance(x, numpy.number)
        if not y_numeric and is_numpy_type(type(y)):
            import numpy
            y_numeric = isinstance(y, numpy.number)

        if x_numeric and y_numeric:
            ty = type(x + y)
            return ty(x), ty(y)

        # Now handle the native python + sage object cases
        # that were not taken care of above.
        elif x_numeric:
            try:
                sage_parent = py_scalar_parent(type(x))
                if sage_parent is None or sage_parent.has_coerce_map_from(yp):
                    return x, x.__class__(y)
                else:
                    return self.canonical_coercion(sage_parent(x), y)
            except (TypeError, ValueError):
                self._record_exception()

        elif y_numeric:
            try:
                sage_parent = py_scalar_parent(type(y))
                if sage_parent is None or sage_parent.has_coerce_map_from(xp):
                    return y.__class__(x), y
                else:
                    return self.canonical_coercion(x, sage_parent(y))
            except (TypeError, ValueError):
                self._record_exception()

        # See if the non-objects define a _sage_ method.
        if not isinstance(x, SageObject) or not isinstance(y, SageObject):
            try:
                x = x._sage_()
                y = y._sage_()
            except AttributeError:
                self._record_exception()
            else:
                return self.canonical_coercion(x, y)

        # Allow coercion of 0 even if no coercion from Z
        if (x_numeric or is_Integer(x)) and not x and type(yp) is not type:
            try:
                return yp(0), y
            except Exception:
                self._record_exception()

        if (y_numeric or is_Integer(y)) and not y and type(xp) is not type:
            try:
                return x, xp(0)
            except Exception:
                self._record_exception()

        raise TypeError("no common canonical parent for objects with parents: '%s' and '%s'"%(xp, yp))


    cpdef coercion_maps(self, R, S):
        r"""
        Give two parents `R` and `S`, return a pair of coercion maps
        `f: R \rightarrow Z` and `g: S \rightarrow Z` , if such a `Z`
        can be found.

        In the (common) case that `R=Z` or `S=Z` then ``None`` is returned
        for `f` or `g` respectively rather than constructing (and subsequently
        calling) the identity morphism.

        If no suitable `f, g` can be found, a single ``None`` is returned.
        This result is cached.

        .. NOTE::

            By :trac:`14711`, coerce maps should be copied when using them
            outside of the coercion system, because they may become defunct
            by garbage collection.

        EXAMPLES::

            sage: cm = sage.structure.element.get_coercion_model()
            sage: f, g = cm.coercion_maps(ZZ, QQ)
            sage: print(copy(f))
            Natural morphism:
              From: Integer Ring
              To:   Rational Field
            sage: print(g)
            None

            sage: ZZx = ZZ['x']
            sage: f, g = cm.coercion_maps(ZZx, QQ)
            sage: print(f)
            (map internal to coercion system -- copy before use)
            Ring morphism:
              From: Univariate Polynomial Ring in x over Integer Ring
              To:   Univariate Polynomial Ring in x over Rational Field
            sage: print(g)
            (map internal to coercion system -- copy before use)
            Polynomial base injection morphism:
              From: Rational Field
              To:   Univariate Polynomial Ring in x over Rational Field

            sage: K = GF(7)
            sage: cm.coercion_maps(QQ, K) is None
            True

        Note that to break symmetry, if there is a coercion map in both
        directions, the parent on the left is used::

            sage: V = QQ^3
            sage: W = V.__class__(QQ, 3)
            sage: V == W
            True
            sage: V is W
            False
            sage: cm = sage.structure.element.get_coercion_model()
            sage: cm.coercion_maps(V, W)
            (None, (map internal to coercion system -- copy before use)
             Coercion map:
               From: Vector space of dimension 3 over Rational Field
               To:   Vector space of dimension 3 over Rational Field)
            sage: cm.coercion_maps(W, V)
            (None, (map internal to coercion system -- copy before use)
             Coercion map:
               From: Vector space of dimension 3 over Rational Field
               To:   Vector space of dimension 3 over Rational Field)
            sage: v = V([1,2,3])
            sage: w = W([1,2,3])
            sage: parent(v+w) is V
            True
            sage: parent(w+v) is W
            True

        TESTS:

        We check that with :trac:`14058`, parents are still eligible for
        garbage collection after being involved in binary operations::

            sage: import gc
            sage: T=type(GF(2))
            sage: gc.collect() #random
            852
            sage: N0=len(list(o for o in gc.get_objects() if type(o) is T))
            sage: L=[ZZ(1)+GF(p)(1) for p in prime_range(2,50)]
            sage: N1=len(list(o for o in gc.get_objects() if type(o) is T))
            sage: N1 > N0
            True
            sage: del L
            sage: gc.collect() #random
            3939
            sage: N2=len(list(o for o in gc.get_objects() if type(o) is T))
            sage: N2-N0
            0

        """
        try:
            refs = self._coercion_maps.get(R, S, None)
            if refs is None:
                return None
            R_map_ref, S_map_ref = refs
            if R_map_ref is None:
                S_map = <object>PyWeakref_GET_OBJECT(S_map_ref)
                if S_map is not None:
                    return None, S_map
            elif S_map_ref is None:
                R_map = <object>PyWeakref_GET_OBJECT(R_map_ref)
                if R_map is not None:
                    return R_map, None
            else:
                R_map = <object>PyWeakref_GET_OBJECT(R_map_ref)
                S_map = <object>PyWeakref_GET_OBJECT(S_map_ref)
                if R_map is not None and S_map is not None:
                    return R_map, S_map
        except KeyError:
            pass
        homs = self.discover_coercion(R, S)
        if 0:
            # This breaks too many things that are going to change
            # in the new coercion model anyways.
            # COERCE TODO: Enable it then.
            homs = self.verify_coercion_maps(R, S, homs)
        else:
            if homs is not None:
                x_map, y_map = homs
                if x_map is not None and not isinstance(x_map, Map):
                    raise RuntimeError("BUG in coercion model: coerce_map_from must return a Map")
                if y_map is not None and not isinstance(y_map, Map):
                    raise RuntimeError("BUG in coercion model: coerce_map_from must return a Map")
        if homs is None:
            refs = None
            swap = None
        else:
            R_map, S_map = homs
            R_map_ref = None if R_map is None else PyWeakref_NewRef(R_map, None)
            S_map_ref = None if S_map is None else PyWeakref_NewRef(S_map, None)
            refs = R_map_ref, S_map_ref
            if R_map is None and isinstance(S, Parent) and (<Parent>S).has_coerce_map_from(R):
                swap = None, PyWeakref_NewRef((<Parent>S).coerce_map_from(R), None)
            else:
                swap = S_map_ref, R_map_ref
        self._coercion_maps.set(R, S, None, refs)
        self._coercion_maps.set(S, R, None, swap)
        return homs

    cpdef verify_coercion_maps(self, R, S, homs, bint fix=False):
        """
        Make sure this is a valid pair of homomorphisms from R and S to a common parent.
        This function is used to protect the user against buggy parents.

        EXAMPLES::

            sage: cm = sage.structure.element.get_coercion_model()
            sage: homs = QQ.coerce_map_from(ZZ), None
            sage: cm.verify_coercion_maps(ZZ, QQ, homs) == homs
            True
            sage: homs = QQ.coerce_map_from(ZZ), RR.coerce_map_from(QQ)
            sage: cm.verify_coercion_maps(ZZ, QQ, homs) == homs
            Traceback (most recent call last):
            ...
            RuntimeError: ('BUG in coercion model, codomains must be identical', Natural morphism:
              From: Integer Ring
              To:   Rational Field, Generic map:
              From: Rational Field
              To:   Real Field with 53 bits of precision)
        """
        if homs is None:
            return None
        cdef Map x_map, y_map
        R_map, S_map = homs
        if isinstance(R, type):
            R = Set_PythonType(R)
        elif isinstance(S, type):
            S = Set_PythonType(S)
        if R_map is None:
            R_map = IdentityMorphism(R)
        elif S_map is None:
            S_map = IdentityMorphism(S)
        # Make sure the domains are correct
        if R_map.domain() is not R:
            if fix:
                connecting = R_map.domain()._internal_coerce_map_from(R)
                if connecting is not None:
                    R_map = R_map * connecting
            if R_map.domain() is not R:
                raise RuntimeError("BUG in coercion model, left domain must be original parent", R, R_map)
        if S_map is not None and S_map.domain() is not S:
            if fix:
                connecting = S_map.domain()._internal_coerce_map_from(S)
                if connecting is not None:
                    S_map = S_map * connecting
            if S_map.domain() is not S:
                raise RuntimeError("BUG in coercion model, right domain must be original parent", S, S_map)
        # Make sure the codomains are correct
        if R_map.codomain() is not S_map.codomain():
            if fix:
                connecting = R_map.codomain()._internal_coerce_map_from(S_map.codomain())
                if connecting is not None:
                    S_map = connecting * S_map
                else:
                    connecting = S_map.codomain()._internal_coerce_map_from(R_map.codomain())
                    if connecting is not None:
                        R_map = connecting * R_map
            if R_map.codomain() is not S_map.codomain():
                raise RuntimeError("BUG in coercion model, codomains must be identical", R_map, S_map)
        if isinstance(R_map, IdentityMorphism):
            R_map = None
        elif isinstance(S_map, IdentityMorphism):
            S_map = None
        return R_map, S_map


    cpdef discover_coercion(self, R, S):
        """
        This actually implements the finding of coercion maps as described in
        the ``coercion_maps`` method.

        EXAMPLES::

            sage: cm = sage.structure.element.get_coercion_model()

        If R is S, then two identity morphisms suffice::

            sage: cm.discover_coercion(SR, SR)
            (None, None)

        If there is a coercion map either direction, use that::

            sage: cm.discover_coercion(ZZ, QQ)
            ((map internal to coercion system -- copy before use)
            Natural morphism:
              From: Integer Ring
              To:   Rational Field, None)
            sage: cm.discover_coercion(RR, QQ)
            (None, (map internal to coercion system -- copy before use)
             Generic map:
              From: Rational Field
              To:   Real Field with 53 bits of precision)

        Otherwise, try and compute an appropriate cover::

            sage: ZZxy = ZZ['x,y']
            sage: cm.discover_coercion(ZZxy, RDF)
            ((map internal to coercion system -- copy before use)
            Call morphism:
              From: Multivariate Polynomial Ring in x, y over Integer Ring
              To:   Multivariate Polynomial Ring in x, y over Real Double Field,
             (map internal to coercion system -- copy before use)
             Polynomial base injection morphism:
              From: Real Double Field
              To:   Multivariate Polynomial Ring in x, y over Real Double Field)

        Sometimes there is a reasonable "cover," but no canonical coercion::

            sage: sage.categories.pushout.pushout(QQ, QQ^3)
            Vector space of dimension 3 over Rational Field
            sage: print(cm.discover_coercion(QQ, QQ^3))
            None
        """
        from sage.categories.homset import Hom
        if R is S:
            return None, None

        # See if there is a natural coercion from R to S
        if isinstance(R, Parent):
            mor = (<Parent>R)._internal_coerce_map_from(S)
            if mor is not None:
                return None, mor

        # See if there is a natural coercion from S to R
        if isinstance(S, Parent):
            mor = (<Parent>S)._internal_coerce_map_from(R)
            if mor is not None:
                return mor, None

        # Try base extending
        if isinstance(R, Parent) and isinstance(S, Parent):
            from sage.categories.pushout import pushout
            try:
                Z = pushout(R, S)
                coerce_R = Z._internal_coerce_map_from(R)
                coerce_S = Z._internal_coerce_map_from(S)
                if coerce_R is None:
                    raise TypeError("No coercion from %s to pushout %s" % (R, Z))
                if coerce_S is None:
                    raise TypeError("No coercion from %s to pushout %s" % (S, Z))
                return coerce_R, coerce_S
            except Exception:
                self._record_exception()

        return None

    cpdef get_action(self, R, S, op=mul, r=None, s=None):
        """
        Get the action of R on S or S on R associated to the operation op.

        EXAMPLES::

            sage: cm = sage.structure.element.get_coercion_model()
            sage: ZZx = ZZ['x']
            sage: cm.get_action(ZZx, ZZ, operator.mul)
            Right scalar multiplication by Integer Ring on Univariate Polynomial Ring in x over Integer Ring
            sage: cm.get_action(ZZx, QQ, operator.mul)
            Right scalar multiplication by Rational Field on Univariate Polynomial Ring in x over Integer Ring
            sage: QQx = QQ['x']
            sage: cm.get_action(QQx, int, operator.mul)
            Right scalar multiplication by Integer Ring on Univariate Polynomial Ring in x over Rational Field
            with precomposition on right by Native morphism:
              From: Set of Python objects of class 'int'
              To:   Integer Ring

            sage: A = cm.get_action(QQx, ZZ, operator.truediv); A
            Right inverse action by Rational Field on Univariate Polynomial Ring in x over Rational Field
            with precomposition on right by Natural morphism:
              From: Integer Ring
              To:   Rational Field
            sage: x = QQx.gen()
            sage: A(x+10, 5)
            1/5*x + 2
        """
        try:
            return self._action_maps.get(R, S, op)
        except KeyError:
            pass
        action = self.discover_action(R, S, op, r, s)
        action = self.verify_action(action, R, S, op)
        self._action_maps.set(R, S, op, action)
        return action

    cpdef verify_action(self, action, R, S, op, bint fix=True):
        r"""
        Verify that ``action`` takes an element of R on the left and S
        on the right, raising an error if not.

        This is used for consistency checking in the coercion model.

        EXAMPLES::

            sage: R.<x> = ZZ['x']
            sage: cm = sage.structure.element.get_coercion_model()
            sage: cm.verify_action(R.get_action(QQ), R, QQ, operator.mul)
            Right scalar multiplication by Rational Field on Univariate Polynomial Ring in x over Integer Ring
            sage: cm.verify_action(R.get_action(QQ), RDF, R, operator.mul)
            Traceback (most recent call last):
            ...
            RuntimeError: There is a BUG in the coercion model:
                Action found for R <built-in function mul> S does not have the correct domains
                R = Real Double Field
                S = Univariate Polynomial Ring in x over Integer Ring
                (should be Univariate Polynomial Ring in x over Integer Ring, Rational Field)
                action = Right scalar multiplication by Rational Field on
                Univariate Polynomial Ring in x over Integer Ring (<class 'sage.structure.coerce_actions.RightModuleAction'>)
        """
        if action is None:
            return action
        cdef bint ok = True
        try:
            if action.left_domain() is not R:
                ok &= isinstance(R, type) and action.left_domain()._type is R
            if action.right_domain() is not S:
                ok &= isinstance(S, type) and action.right_domain()._type is S
        except AttributeError:
            ok = False
        if not ok:
            if isinstance(R, type):
                R = Set_PythonType(R)
            if isinstance(S, type):
                S = Set_PythonType(S)

            # Non-unique parents
            if fix and action.left_domain() is not R and action.left_domain() == R:
                action = PrecomposedAction(action, action.left_domain()._internal_coerce_map_from(R), None)
            if fix and action.right_domain() is not S and action.right_domain() == S:
                action = PrecomposedAction(action, None, action.right_domain()._internal_coerce_map_from(S))

            if action.left_domain() is not R or action.right_domain() is not S:
                raise RuntimeError("""There is a BUG in the coercion model:
                Action found for R %s S does not have the correct domains
                R = %s
                S = %s
                (should be %s, %s)
                action = %s (%s)
                """ % (op, R, S, action.left_domain(), action.right_domain(), action, type(action)))

        return action

    cpdef discover_action(self, R, S, op, r=None, s=None):
        """
        INPUT:

        - ``R`` - the left Parent (or type)
        - ``S`` - the right Parent (or type)
        - ``op`` - the operand, typically an element of the operator module
        - ``r`` - (optional) element of R
        - ``s`` - (optional) element of S.

        OUTPUT:

        - An action A such that s op r is given by A(s,r).

        The steps taken are illustrated below.

        EXAMPLES::

            sage: P.<x> = ZZ['x']
            sage: P.get_action(ZZ)
            Right scalar multiplication by Integer Ring on Univariate Polynomial Ring in x over Integer Ring
            sage: ZZ.get_action(P) is None
            True
            sage: cm = sage.structure.element.get_coercion_model()

        If R or S is a Parent, ask it for an action by/on R::

            sage: cm.discover_action(ZZ, P, operator.mul)
            Left scalar multiplication by Integer Ring on Univariate Polynomial Ring in x over Integer Ring

        If R or S a type, recursively call get_action with the Sage versions of R and/or S::

            sage: cm.discover_action(P, int, operator.mul)
            Right scalar multiplication by Integer Ring on Univariate Polynomial Ring in x over Integer Ring
            with precomposition on right by Native morphism:
              From: Set of Python objects of class 'int'
              To:   Integer Ring

        If op is division, look for action on right by inverse::

            sage: cm.discover_action(P, ZZ, operator.truediv)
            Right inverse action by Rational Field on Univariate Polynomial Ring in x over Integer Ring
            with precomposition on right by Natural morphism:
              From: Integer Ring
              To:   Rational Field

        Check that :trac:`17740` is fixed::

            sage: R = GF(5)['x']
            sage: cm.discover_action(R, ZZ, operator.truediv)
            Right inverse action by Finite Field of size 5 on Univariate Polynomial Ring in x over Finite Field of size 5
            with precomposition on right by Natural morphism:
              From: Integer Ring
              To:   Finite Field of size 5
            sage: cm.bin_op(R.gen(), 7, operator.truediv).parent()
            Univariate Polynomial Ring in x over Finite Field of size 5

        Check that :trac:`18221` is fixed::

            sage: F.<x> = FreeAlgebra(QQ)
            sage: x / 2
            1/2*x
            sage: cm.discover_action(F, ZZ, operator.truediv)
            Right inverse action by Rational Field on Free Algebra on 1 generators (x,) over Rational Field
            with precomposition on right by Natural morphism:
              From: Integer Ring
              To:   Rational Field
        """
        if isinstance(R, Parent):
            action = (<Parent>R).get_action(S, op, True, r, s)
            if action is not None:
                return action

        if isinstance(S, Parent):
            action = (<Parent>S).get_action(R, op, False, s, r)
            if action is not None:
                return action

        if type(R) is type:
            sageR = py_scalar_parent(R)
            if sageR is not None:
                action = self.get_action(sageR, S, op, s=s)
                if action is not None:
                    return PrecomposedAction(action, sageR._internal_coerce_map_from(R), None)

        if type(S) is type:
            sageS = py_scalar_parent(S)
            if sageS is not None:
                action = self.get_action(R, sageS, op, r=r)
                if action is not None:
                    return PrecomposedAction(action, None, sageS._internal_coerce_map_from(S))

        if op is truediv:
            # Division on right is the same acting on right by inverse, if it is so defined.
            right_mul = None
            try:
                right_mul = self.get_action(R, S, mul)
            except NotImplementedError:
                self._record_exception()

            if right_mul is not None and not right_mul.is_left():
                try:
                    action = ~right_mul
                    if action.right_domain() != S:
                        action = PrecomposedAction(action, None,
                                                   action.right_domain()._internal_coerce_map_from(S))
                    return action
                except TypeError: # action may not be invertible
                    self._record_exception()

            # It's possible an action is defined on the fraction field itself.
            try:
                K = S._pseudo_fraction_field()
            except AttributeError:
                pass
            else:
                if K is not S:
                    try:
                        right_mul = self.get_action(R, K, mul)
                    except NotImplementedError:
                        self._record_exception()

                    if right_mul is not None and not right_mul.is_left():
                        try:
                            return PrecomposedAction(~right_mul, None, K.coerce_map_from(S))
                        except TypeError: # action may not be invertible
                            self._record_exception()

        return None

    cpdef richcmp(self, x, y, int op):
        """
        Given two arbitrary objects ``x`` and ``y``, coerce them to
        a common parent and compare them using rich comparison operator
        ``op``.

        EXAMPLES::

            sage: from sage.structure.element import get_coercion_model
            sage: from sage.structure.richcmp import op_LT, op_LE, op_EQ, op_NE, op_GT, op_GE
            sage: richcmp = get_coercion_model().richcmp
            sage: richcmp(None, None, op_EQ)
            True
            sage: richcmp(None, 1, op_LT)
            True
            sage: richcmp("hello", None, op_LE)
            False
            sage: richcmp(-1, 1, op_GE)
            False
            sage: richcmp(int(1), float(2), op_GE)
            False

        If there is no coercion, we only support ``==`` and ``!=``::

            sage: x = QQ.one(); y = GF(2).one()
            sage: richcmp(x, y, op_EQ)
            False
            sage: richcmp(x, y, op_NE)
            True
            sage: richcmp(x, y, op_GT)
            Traceback (most recent call last):
            ...
            TypeError: unsupported operand parent(s) for >: 'Rational Field' and 'Finite Field of size 2'

        We support non-Sage types with the usual Python convention::

            sage: class AlwaysEqual(object):
            ....:     def __eq__(self, other):
            ....:         return True
            sage: x = AlwaysEqual()
            sage: x == 1
            True
            sage: 1 == x
            True
        """
        # Some very special cases
        if x is None or x is Ellipsis:
            return rich_to_bool(op, 0 if x is y else -1)
        if y is None or y is Ellipsis:
            return rich_to_bool(op, 0 if x is y else 1)

        cdef bint y_is_Element = isinstance(y, Element)

        # Check for manual __richcmp__ override (only on y since
        # x.__richcmp__ would already have been called)
        if y_is_Element:
            if (<Element>y)._parent.get_flag(Parent_richcmp_element_without_coercion):
                return Py_TYPE(y).tp_richcompare(y, x, revop(op))

        # Coerce to a common parent
        try:
            x, y = self.canonical_coercion(x, y)
        except (TypeError, NotImplementedError):
            pass
        else:
            # The common parent should not be one which explicitly
            # asked to *not* use coercion for comparisons.
            assert not (isinstance(x, Element) and
                (<Element>x)._parent.get_flag(Parent_richcmp_element_without_coercion))
            return PyObject_RichCompare(x, y, op)

        # Comparing with coercion didn't work, try something else.

        # Try y.__richcmp__(x, revop) where revop is the reversed
        # operation (<= becomes >=).
        # This only makes sense when y is not a Sage Element, otherwise
        # we would end up trying the same coercion again.
        if not y_is_Element and Py_TYPE(y).tp_richcompare:
            res = Py_TYPE(y).tp_richcompare(y, x, revop(op))
            if res is not NotImplemented:
                return res

        # At this point, we have 2 objects which cannot be coerced to
        # a common parent. So we assume that they are not equal.
        if op == Py_EQ:
            return False
        if op == Py_NE:
            return True

        # It does not make sense to compare x and y with an inequality,
        # so we raise an exception.
        if op == Py_LT:
            raise bin_op_exception('<', x, y)
        elif op == Py_LE:
            raise bin_op_exception('<=', x, y)
        elif op == Py_GT:
            raise bin_op_exception('>', x, y)
        else:
            raise bin_op_exception('>=', x, y)

    def _coercion_error(self, x, x_map, x_elt, y, y_map, y_elt):
        """
        This function is only called when someone has incorrectly implemented
        a user-defined part of the coercion system (usually, a morphism).

        EXAMPLES::

            sage: cm = sage.structure.element.get_coercion_model()
            sage: cm._coercion_error('a', 'f', 'f(a)', 'b', 'g', 'g(b)')
            Traceback (most recent call last):
            ...
            RuntimeError: There is a bug in the coercion code in Sage.
            Both x (='f(a)') and y (='g(b)') are supposed to have identical parents but they don't.
            In fact, x has parent '<class 'str'>'
            whereas y has parent '<class 'str'>'
            Original elements 'a' (parent <class 'str'>) and 'b' (parent <class 'str'>) and maps
            <class 'str'> 'f'
            <class 'str'> 'g'
        """
        raise RuntimeError("""There is a bug in the coercion code in Sage.
Both x (=%r) and y (=%r) are supposed to have identical parents but they don't.
In fact, x has parent '%s'
whereas y has parent '%s'
Original elements %r (parent %s) and %r (parent %s) and maps
%s %r
%s %r""" % (x_elt, y_elt, parent(x_elt), parent(y_elt),
            x, parent(x), y, parent(y),
            type(x_map), x_map, type(y_map), y_map))


coercion_model = CoercionModel()



"""
Debug options for the :mod:`sage.structure` modules

EXAMPLES::

    sage: from sage.structure.debug_options import debug
    sage: debug.unique_parent_warnings
    False
    sage: debug.refine_category_hash_check
    True
"""

#*****************************************************************************
#       Copyright (C) 2013 Robert Bradshaw <robertwb@gmail.com>
#                          William Stein <wstein@gmail.com>
#                          Volker Braun <vbraun.name@gmail.com>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#                  http://www.gnu.org/licenses/
#*****************************************************************************


cdef class DebugOptions_class:
    def __cinit__(self):
        """
        Initializer for the debug options

        TESTS::

            sage: from sage.structure.debug_options import debug
            sage: type(debug)
            <... 'sage.structure.debug_options.DebugOptions_class'>
        """
        self.unique_parent_warnings = False
        # This one will be enabled during doctests
        self.refine_category_hash_check = False


cdef DebugOptions_class debug = DebugOptions_class()

# Since "debug" is declared with a type, it can only be cimported at
# the Cython level, not imported in plain Python. So we add it to the
# globals manually. We need to hack into Cython internals for this
# since Sage is compiled with the old_style_globals option.
from cpython.object cimport PyObject
cdef extern from *:
    PyObject* __pyx_d

(<object>__pyx_d)["debug"] = debug



# -*- coding: utf-8 -*-
r"""
Features for testing the presence of ``fes``
"""

from . import CythonFeature, PythonModule
from .join_feature import JoinFeature


TEST_CODE = """
# distutils: libraries=fes

from libc.stdint cimport uint64_t
cdef extern from "<fes_interface.h>":
    ctypedef int (*solution_callback_t)(void *, uint64_t)
    void exhaustive_search_wrapper(int n, int n_eqs, int degree, int ***coeffs, solution_callback_t callback, void* callback_state, int verbose)

solutions = 0

class InternalState:
    verbose = False
    sols = []
    max_sols = 0

cdef int report_solution(void *_state, uint64_t i):
    global solutions
    solutions += 1
    return 0

sig_on()
cdef int ***coeffs = <int ***> sig_calloc(1, sizeof(int **))
coeffs[0] = <int **> sig_calloc(3, sizeof(int *))
coeffs[0][0] = <int *> sig_calloc(1, sizeof(int))
coeffs[0][1] = <int *> sig_calloc(2, sizeof(int))
coeffs[0][2] = <int *> sig_calloc(1, sizeof(int))
coeffs[0][2][0] = 1  # x*y = 0
internal_state = InternalState()

exhaustive_search_wrapper(2, 1, 2, coeffs, report_solution, <void *>internal_state, 0)

sig_free(coeffs[0][2])
sig_free(coeffs[0][1])
sig_free(coeffs[0][0])
sig_free(coeffs[0])
sig_free(coeffs)
sig_off()

if solutions != 3:
    raise AssertionError("libFES did not find three solutions for x*y = 0")
"""


class LibFESLibrary(CythonFeature):
    r"""
    A :class:`~sage.features.Feature` which describes whether the FES library
    is present and functional.

    EXAMPLES::

        sage: from sage.features.fes import LibFESLibrary
        sage: LibFESLibrary().require()  # optional - fes
    """
    def __init__(self):
        r"""
        TESTS::

            sage: from sage.features.fes import LibFESLibrary
            sage: isinstance(LibFESLibrary(), LibFESLibrary)
            True
        """
        CythonFeature.__init__(self, "LibFES", test_code=TEST_CODE, spkg="fes",
                               url="http://www.lifl.fr/~bouillag/fes/")


class LibFES(JoinFeature):
    r"""
    A :class:`~sage.features.Feature` which describes whether the :mod:`sage.libs.fes`
    module has been enabled for this build of Sage and is functional.

    EXAMPLES::

        sage: from sage.features.fes import LibFES
        sage: LibFES().require()  # optional - fes
    """
    def __init__(self):
        r"""
        TESTS::

            sage: from sage.features.fes import LibFES
            sage: isinstance(LibFES(), LibFES)
            True
        """
        JoinFeature.__init__(self, 'fes',
                             [PythonModule("sage.libs.fes")],
                             spkg="fes",
                             url="http://www.lifl.fr/~bouillag/fes/")
